# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['ServerArgs', 'Server']

@pulumi.input_type
class ServerArgs:
    def __init__(__self__, *,
                 offer: pulumi.Input[_builtins.str],
                 cloud_init: Optional[pulumi.Input[_builtins.str]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 hostname: Optional[pulumi.Input[_builtins.str]] = None,
                 install_config_afterward: Optional[pulumi.Input[_builtins.bool]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 options: Optional[pulumi.Input[Sequence[pulumi.Input['ServerOptionArgs']]]] = None,
                 os: Optional[pulumi.Input[_builtins.str]] = None,
                 partitioning: Optional[pulumi.Input[_builtins.str]] = None,
                 password: Optional[pulumi.Input[_builtins.str]] = None,
                 password_wo: Optional[pulumi.Input[_builtins.str]] = None,
                 password_wo_version: Optional[pulumi.Input[_builtins.int]] = None,
                 private_ips: Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateIpArgs']]]] = None,
                 private_networks: Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateNetworkArgs']]]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 protected: Optional[pulumi.Input[_builtins.bool]] = None,
                 reinstall_on_config_changes: Optional[pulumi.Input[_builtins.bool]] = None,
                 service_password: Optional[pulumi.Input[_builtins.str]] = None,
                 service_password_wo: Optional[pulumi.Input[_builtins.str]] = None,
                 service_password_wo_version: Optional[pulumi.Input[_builtins.int]] = None,
                 service_user: Optional[pulumi.Input[_builtins.str]] = None,
                 ssh_key_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 user: Optional[pulumi.Input[_builtins.str]] = None,
                 zone: Optional[pulumi.Input[_builtins.str]] = None):
        """
        The set of arguments for constructing a Server resource.
        :param pulumi.Input[_builtins.str] offer: The offer UUID of the baremetal server.
               Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-servers-get-a-specific-elastic-metal-server) to find the right offer.
               
               > **Important:** Updates to `offer` will recreate the server.
               > **Important**  If you migrate to a monthly subscription, you cannot downgrade back to an hourly plan. Ensure that the monthly plan meets your needs before making the switch.
        :param pulumi.Input[_builtins.str] cloud_init: Configuration data to pass to cloud-init such as a YAML cloud config or a user-data script. Accepts either a string containing the content or a path to a file (for example `file("cloud-init.yml")`). Max length: 127998 characters. Updates to `cloud_init` will update the server user-data via the API and do not trigger a reinstall; however, a reboot of the server is required for the OS to re-run cloud-init and apply the changes. Only supported for Offers that have cloud-init enabled. You can check available offers with `scw baremetal list offers` command.
        :param pulumi.Input[_builtins.str] description: A description for the server.
        :param pulumi.Input[_builtins.str] hostname: The hostname of the server.
        :param pulumi.Input[_builtins.bool] install_config_afterward: If True, this boolean allows to create a server without the install config if you want to provide it later.
        :param pulumi.Input[_builtins.str] name: The name of the server.
        :param pulumi.Input[Sequence[pulumi.Input['ServerOptionArgs']]] options: The options to enable on the server.
               > The `options` block supports:
        :param pulumi.Input[_builtins.str] os: The UUID of the os to install on the server.
               Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-os-list-available-oses) to find the right OS ID.
               > **Important:** Updates to `os` will reinstall the server.
        :param pulumi.Input[_builtins.str] partitioning: The partitioning schema in JSON format
        :param pulumi.Input[_builtins.str] password: Password used for the installation. May be required depending on used os. Only one of `password` or `password_wo` should be specified.
        :param pulumi.Input[_builtins.str] password_wo: **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        :param pulumi.Input[_builtins.int] password_wo_version: The version of the write-only password. To update the `password_wo`, you must also update the `password_wo_version`.
        :param pulumi.Input[Sequence[pulumi.Input['ServerPrivateIpArgs']]] private_ips: The list of private IPv4 and IPv6 addresses associated with the resource.
        :param pulumi.Input[Sequence[pulumi.Input['ServerPrivateNetworkArgs']]] private_networks: The private networks to attach to the server. For more information, see [the documentation](https://www.scaleway.com/en/docs/compute/elastic-metal/how-to/use-private-networks/)
        :param pulumi.Input[_builtins.str] project_id: `project_id`) The ID of the project the server is associated with.
        :param pulumi.Input[_builtins.bool] protected: Set to true to activate server protection option.
        :param pulumi.Input[_builtins.bool] reinstall_on_config_changes: If True, this boolean allows to reinstall the server on install config changes.
               > **Important:** Updates to `ssh_key_ids`, `user`, `password`, `service_user` or `service_password` will not take effect on the server, it requires to reinstall it. To do so please set 'reinstall_on_config_changes' argument to true.
        :param pulumi.Input[_builtins.str] service_password: Password used for the service to install. May be required depending on used os. Only one of `service_password` or `service_password_wo` should be specified.
        :param pulumi.Input[_builtins.str] service_password_wo: **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        :param pulumi.Input[_builtins.int] service_password_wo_version: The version of the write-only service password. To update the `service_password_wo`, you must also update the `service_password_wo_version`.
        :param pulumi.Input[_builtins.str] service_user: User used for the service to install.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] ssh_key_ids: List of SSH keys allowed to connect to the server.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags associated with the server.
        :param pulumi.Input[_builtins.str] user: User used for the installation.
        :param pulumi.Input[_builtins.str] zone: `zone`) The zone in which the server should be created.
        """
        pulumi.set(__self__, "offer", offer)
        if cloud_init is not None:
            pulumi.set(__self__, "cloud_init", cloud_init)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if hostname is not None:
            pulumi.set(__self__, "hostname", hostname)
        if install_config_afterward is not None:
            pulumi.set(__self__, "install_config_afterward", install_config_afterward)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if options is not None:
            pulumi.set(__self__, "options", options)
        if os is not None:
            pulumi.set(__self__, "os", os)
        if partitioning is not None:
            pulumi.set(__self__, "partitioning", partitioning)
        if password is not None:
            pulumi.set(__self__, "password", password)
        if password_wo is not None:
            pulumi.set(__self__, "password_wo", password_wo)
        if password_wo_version is not None:
            pulumi.set(__self__, "password_wo_version", password_wo_version)
        if private_ips is not None:
            pulumi.set(__self__, "private_ips", private_ips)
        if private_networks is not None:
            pulumi.set(__self__, "private_networks", private_networks)
        if project_id is not None:
            pulumi.set(__self__, "project_id", project_id)
        if protected is not None:
            pulumi.set(__self__, "protected", protected)
        if reinstall_on_config_changes is not None:
            pulumi.set(__self__, "reinstall_on_config_changes", reinstall_on_config_changes)
        if service_password is not None:
            pulumi.set(__self__, "service_password", service_password)
        if service_password_wo is not None:
            pulumi.set(__self__, "service_password_wo", service_password_wo)
        if service_password_wo_version is not None:
            pulumi.set(__self__, "service_password_wo_version", service_password_wo_version)
        if service_user is not None:
            pulumi.set(__self__, "service_user", service_user)
        if ssh_key_ids is not None:
            pulumi.set(__self__, "ssh_key_ids", ssh_key_ids)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if user is not None:
            pulumi.set(__self__, "user", user)
        if zone is not None:
            pulumi.set(__self__, "zone", zone)

    @_builtins.property
    @pulumi.getter
    def offer(self) -> pulumi.Input[_builtins.str]:
        """
        The offer UUID of the baremetal server.
        Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-servers-get-a-specific-elastic-metal-server) to find the right offer.

        > **Important:** Updates to `offer` will recreate the server.
        > **Important**  If you migrate to a monthly subscription, you cannot downgrade back to an hourly plan. Ensure that the monthly plan meets your needs before making the switch.
        """
        return pulumi.get(self, "offer")

    @offer.setter
    def offer(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "offer", value)

    @_builtins.property
    @pulumi.getter(name="cloudInit")
    def cloud_init(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Configuration data to pass to cloud-init such as a YAML cloud config or a user-data script. Accepts either a string containing the content or a path to a file (for example `file("cloud-init.yml")`). Max length: 127998 characters. Updates to `cloud_init` will update the server user-data via the API and do not trigger a reinstall; however, a reboot of the server is required for the OS to re-run cloud-init and apply the changes. Only supported for Offers that have cloud-init enabled. You can check available offers with `scw baremetal list offers` command.
        """
        return pulumi.get(self, "cloud_init")

    @cloud_init.setter
    def cloud_init(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "cloud_init", value)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A description for the server.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter
    def hostname(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The hostname of the server.
        """
        return pulumi.get(self, "hostname")

    @hostname.setter
    def hostname(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "hostname", value)

    @_builtins.property
    @pulumi.getter(name="installConfigAfterward")
    def install_config_afterward(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        If True, this boolean allows to create a server without the install config if you want to provide it later.
        """
        return pulumi.get(self, "install_config_afterward")

    @install_config_afterward.setter
    def install_config_afterward(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "install_config_afterward", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the server.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter
    def options(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ServerOptionArgs']]]]:
        """
        The options to enable on the server.
        > The `options` block supports:
        """
        return pulumi.get(self, "options")

    @options.setter
    def options(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ServerOptionArgs']]]]):
        pulumi.set(self, "options", value)

    @_builtins.property
    @pulumi.getter
    def os(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The UUID of the os to install on the server.
        Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-os-list-available-oses) to find the right OS ID.
        > **Important:** Updates to `os` will reinstall the server.
        """
        return pulumi.get(self, "os")

    @os.setter
    def os(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "os", value)

    @_builtins.property
    @pulumi.getter
    def partitioning(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The partitioning schema in JSON format
        """
        return pulumi.get(self, "partitioning")

    @partitioning.setter
    def partitioning(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "partitioning", value)

    @_builtins.property
    @pulumi.getter
    def password(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Password used for the installation. May be required depending on used os. Only one of `password` or `password_wo` should be specified.
        """
        return pulumi.get(self, "password")

    @password.setter
    def password(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "password", value)

    @_builtins.property
    @pulumi.getter(name="passwordWo")
    def password_wo(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        """
        return pulumi.get(self, "password_wo")

    @password_wo.setter
    def password_wo(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "password_wo", value)

    @_builtins.property
    @pulumi.getter(name="passwordWoVersion")
    def password_wo_version(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The version of the write-only password. To update the `password_wo`, you must also update the `password_wo_version`.
        """
        return pulumi.get(self, "password_wo_version")

    @password_wo_version.setter
    def password_wo_version(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "password_wo_version", value)

    @_builtins.property
    @pulumi.getter(name="privateIps")
    def private_ips(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateIpArgs']]]]:
        """
        The list of private IPv4 and IPv6 addresses associated with the resource.
        """
        return pulumi.get(self, "private_ips")

    @private_ips.setter
    def private_ips(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateIpArgs']]]]):
        pulumi.set(self, "private_ips", value)

    @_builtins.property
    @pulumi.getter(name="privateNetworks")
    def private_networks(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateNetworkArgs']]]]:
        """
        The private networks to attach to the server. For more information, see [the documentation](https://www.scaleway.com/en/docs/compute/elastic-metal/how-to/use-private-networks/)
        """
        return pulumi.get(self, "private_networks")

    @private_networks.setter
    def private_networks(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateNetworkArgs']]]]):
        pulumi.set(self, "private_networks", value)

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `project_id`) The ID of the project the server is associated with.
        """
        return pulumi.get(self, "project_id")

    @project_id.setter
    def project_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "project_id", value)

    @_builtins.property
    @pulumi.getter
    def protected(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Set to true to activate server protection option.
        """
        return pulumi.get(self, "protected")

    @protected.setter
    def protected(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "protected", value)

    @_builtins.property
    @pulumi.getter(name="reinstallOnConfigChanges")
    def reinstall_on_config_changes(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        If True, this boolean allows to reinstall the server on install config changes.
        > **Important:** Updates to `ssh_key_ids`, `user`, `password`, `service_user` or `service_password` will not take effect on the server, it requires to reinstall it. To do so please set 'reinstall_on_config_changes' argument to true.
        """
        return pulumi.get(self, "reinstall_on_config_changes")

    @reinstall_on_config_changes.setter
    def reinstall_on_config_changes(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "reinstall_on_config_changes", value)

    @_builtins.property
    @pulumi.getter(name="servicePassword")
    def service_password(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Password used for the service to install. May be required depending on used os. Only one of `service_password` or `service_password_wo` should be specified.
        """
        return pulumi.get(self, "service_password")

    @service_password.setter
    def service_password(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "service_password", value)

    @_builtins.property
    @pulumi.getter(name="servicePasswordWo")
    def service_password_wo(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        """
        return pulumi.get(self, "service_password_wo")

    @service_password_wo.setter
    def service_password_wo(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "service_password_wo", value)

    @_builtins.property
    @pulumi.getter(name="servicePasswordWoVersion")
    def service_password_wo_version(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The version of the write-only service password. To update the `service_password_wo`, you must also update the `service_password_wo_version`.
        """
        return pulumi.get(self, "service_password_wo_version")

    @service_password_wo_version.setter
    def service_password_wo_version(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "service_password_wo_version", value)

    @_builtins.property
    @pulumi.getter(name="serviceUser")
    def service_user(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        User used for the service to install.
        """
        return pulumi.get(self, "service_user")

    @service_user.setter
    def service_user(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "service_user", value)

    @_builtins.property
    @pulumi.getter(name="sshKeyIds")
    def ssh_key_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        List of SSH keys allowed to connect to the server.
        """
        return pulumi.get(self, "ssh_key_ids")

    @ssh_key_ids.setter
    def ssh_key_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "ssh_key_ids", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The tags associated with the server.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter
    def user(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        User used for the installation.
        """
        return pulumi.get(self, "user")

    @user.setter
    def user(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "user", value)

    @_builtins.property
    @pulumi.getter
    def zone(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `zone`) The zone in which the server should be created.
        """
        return pulumi.get(self, "zone")

    @zone.setter
    def zone(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "zone", value)


@pulumi.input_type
class _ServerState:
    def __init__(__self__, *,
                 cloud_init: Optional[pulumi.Input[_builtins.str]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 domain: Optional[pulumi.Input[_builtins.str]] = None,
                 hostname: Optional[pulumi.Input[_builtins.str]] = None,
                 install_config_afterward: Optional[pulumi.Input[_builtins.bool]] = None,
                 ips: Optional[pulumi.Input[Sequence[pulumi.Input['ServerIpArgs']]]] = None,
                 ipv4s: Optional[pulumi.Input[Sequence[pulumi.Input['ServerIpv4Args']]]] = None,
                 ipv6s: Optional[pulumi.Input[Sequence[pulumi.Input['ServerIpv6Args']]]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 offer: Optional[pulumi.Input[_builtins.str]] = None,
                 offer_id: Optional[pulumi.Input[_builtins.str]] = None,
                 offer_name: Optional[pulumi.Input[_builtins.str]] = None,
                 options: Optional[pulumi.Input[Sequence[pulumi.Input['ServerOptionArgs']]]] = None,
                 organization_id: Optional[pulumi.Input[_builtins.str]] = None,
                 os: Optional[pulumi.Input[_builtins.str]] = None,
                 os_name: Optional[pulumi.Input[_builtins.str]] = None,
                 partitioning: Optional[pulumi.Input[_builtins.str]] = None,
                 password: Optional[pulumi.Input[_builtins.str]] = None,
                 password_wo: Optional[pulumi.Input[_builtins.str]] = None,
                 password_wo_version: Optional[pulumi.Input[_builtins.int]] = None,
                 private_ips: Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateIpArgs']]]] = None,
                 private_networks: Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateNetworkArgs']]]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 protected: Optional[pulumi.Input[_builtins.bool]] = None,
                 reinstall_on_config_changes: Optional[pulumi.Input[_builtins.bool]] = None,
                 service_password: Optional[pulumi.Input[_builtins.str]] = None,
                 service_password_wo: Optional[pulumi.Input[_builtins.str]] = None,
                 service_password_wo_version: Optional[pulumi.Input[_builtins.int]] = None,
                 service_user: Optional[pulumi.Input[_builtins.str]] = None,
                 ssh_key_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 user: Optional[pulumi.Input[_builtins.str]] = None,
                 zone: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering Server resources.
        :param pulumi.Input[_builtins.str] cloud_init: Configuration data to pass to cloud-init such as a YAML cloud config or a user-data script. Accepts either a string containing the content or a path to a file (for example `file("cloud-init.yml")`). Max length: 127998 characters. Updates to `cloud_init` will update the server user-data via the API and do not trigger a reinstall; however, a reboot of the server is required for the OS to re-run cloud-init and apply the changes. Only supported for Offers that have cloud-init enabled. You can check available offers with `scw baremetal list offers` command.
        :param pulumi.Input[_builtins.str] description: A description for the server.
        :param pulumi.Input[_builtins.str] domain: The domain of the server.
        :param pulumi.Input[_builtins.str] hostname: The hostname of the server.
        :param pulumi.Input[_builtins.bool] install_config_afterward: If True, this boolean allows to create a server without the install config if you want to provide it later.
        :param pulumi.Input[Sequence[pulumi.Input['ServerIpArgs']]] ips: (List of) The IPs of the server.
        :param pulumi.Input[Sequence[pulumi.Input['ServerIpv4Args']]] ipv4s: (List of) The IPv4 addresses of the server.
        :param pulumi.Input[Sequence[pulumi.Input['ServerIpv6Args']]] ipv6s: (List of) The IPv6 addresses of the server.
        :param pulumi.Input[_builtins.str] name: The name of the server.
        :param pulumi.Input[_builtins.str] offer: The offer UUID of the baremetal server.
               Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-servers-get-a-specific-elastic-metal-server) to find the right offer.
               
               > **Important:** Updates to `offer` will recreate the server.
               > **Important**  If you migrate to a monthly subscription, you cannot downgrade back to an hourly plan. Ensure that the monthly plan meets your needs before making the switch.
        :param pulumi.Input[_builtins.str] offer_id: The ID of the offer.
        :param pulumi.Input[_builtins.str] offer_name: The name of the offer.
        :param pulumi.Input[Sequence[pulumi.Input['ServerOptionArgs']]] options: The options to enable on the server.
               > The `options` block supports:
        :param pulumi.Input[_builtins.str] organization_id: The organization ID the server is associated with.
        :param pulumi.Input[_builtins.str] os: The UUID of the os to install on the server.
               Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-os-list-available-oses) to find the right OS ID.
               > **Important:** Updates to `os` will reinstall the server.
        :param pulumi.Input[_builtins.str] os_name: The name of the os.
        :param pulumi.Input[_builtins.str] partitioning: The partitioning schema in JSON format
        :param pulumi.Input[_builtins.str] password: Password used for the installation. May be required depending on used os. Only one of `password` or `password_wo` should be specified.
        :param pulumi.Input[_builtins.str] password_wo: **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        :param pulumi.Input[_builtins.int] password_wo_version: The version of the write-only password. To update the `password_wo`, you must also update the `password_wo_version`.
        :param pulumi.Input[Sequence[pulumi.Input['ServerPrivateIpArgs']]] private_ips: The list of private IPv4 and IPv6 addresses associated with the resource.
        :param pulumi.Input[Sequence[pulumi.Input['ServerPrivateNetworkArgs']]] private_networks: The private networks to attach to the server. For more information, see [the documentation](https://www.scaleway.com/en/docs/compute/elastic-metal/how-to/use-private-networks/)
        :param pulumi.Input[_builtins.str] project_id: `project_id`) The ID of the project the server is associated with.
        :param pulumi.Input[_builtins.bool] protected: Set to true to activate server protection option.
        :param pulumi.Input[_builtins.bool] reinstall_on_config_changes: If True, this boolean allows to reinstall the server on install config changes.
               > **Important:** Updates to `ssh_key_ids`, `user`, `password`, `service_user` or `service_password` will not take effect on the server, it requires to reinstall it. To do so please set 'reinstall_on_config_changes' argument to true.
        :param pulumi.Input[_builtins.str] service_password: Password used for the service to install. May be required depending on used os. Only one of `service_password` or `service_password_wo` should be specified.
        :param pulumi.Input[_builtins.str] service_password_wo: **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        :param pulumi.Input[_builtins.int] service_password_wo_version: The version of the write-only service password. To update the `service_password_wo`, you must also update the `service_password_wo_version`.
        :param pulumi.Input[_builtins.str] service_user: User used for the service to install.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] ssh_key_ids: List of SSH keys allowed to connect to the server.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags associated with the server.
        :param pulumi.Input[_builtins.str] user: User used for the installation.
        :param pulumi.Input[_builtins.str] zone: `zone`) The zone in which the server should be created.
        """
        if cloud_init is not None:
            pulumi.set(__self__, "cloud_init", cloud_init)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if domain is not None:
            pulumi.set(__self__, "domain", domain)
        if hostname is not None:
            pulumi.set(__self__, "hostname", hostname)
        if install_config_afterward is not None:
            pulumi.set(__self__, "install_config_afterward", install_config_afterward)
        if ips is not None:
            pulumi.set(__self__, "ips", ips)
        if ipv4s is not None:
            pulumi.set(__self__, "ipv4s", ipv4s)
        if ipv6s is not None:
            pulumi.set(__self__, "ipv6s", ipv6s)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if offer is not None:
            pulumi.set(__self__, "offer", offer)
        if offer_id is not None:
            pulumi.set(__self__, "offer_id", offer_id)
        if offer_name is not None:
            pulumi.set(__self__, "offer_name", offer_name)
        if options is not None:
            pulumi.set(__self__, "options", options)
        if organization_id is not None:
            pulumi.set(__self__, "organization_id", organization_id)
        if os is not None:
            pulumi.set(__self__, "os", os)
        if os_name is not None:
            pulumi.set(__self__, "os_name", os_name)
        if partitioning is not None:
            pulumi.set(__self__, "partitioning", partitioning)
        if password is not None:
            pulumi.set(__self__, "password", password)
        if password_wo is not None:
            pulumi.set(__self__, "password_wo", password_wo)
        if password_wo_version is not None:
            pulumi.set(__self__, "password_wo_version", password_wo_version)
        if private_ips is not None:
            pulumi.set(__self__, "private_ips", private_ips)
        if private_networks is not None:
            pulumi.set(__self__, "private_networks", private_networks)
        if project_id is not None:
            pulumi.set(__self__, "project_id", project_id)
        if protected is not None:
            pulumi.set(__self__, "protected", protected)
        if reinstall_on_config_changes is not None:
            pulumi.set(__self__, "reinstall_on_config_changes", reinstall_on_config_changes)
        if service_password is not None:
            pulumi.set(__self__, "service_password", service_password)
        if service_password_wo is not None:
            pulumi.set(__self__, "service_password_wo", service_password_wo)
        if service_password_wo_version is not None:
            pulumi.set(__self__, "service_password_wo_version", service_password_wo_version)
        if service_user is not None:
            pulumi.set(__self__, "service_user", service_user)
        if ssh_key_ids is not None:
            pulumi.set(__self__, "ssh_key_ids", ssh_key_ids)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if user is not None:
            pulumi.set(__self__, "user", user)
        if zone is not None:
            pulumi.set(__self__, "zone", zone)

    @_builtins.property
    @pulumi.getter(name="cloudInit")
    def cloud_init(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Configuration data to pass to cloud-init such as a YAML cloud config or a user-data script. Accepts either a string containing the content or a path to a file (for example `file("cloud-init.yml")`). Max length: 127998 characters. Updates to `cloud_init` will update the server user-data via the API and do not trigger a reinstall; however, a reboot of the server is required for the OS to re-run cloud-init and apply the changes. Only supported for Offers that have cloud-init enabled. You can check available offers with `scw baremetal list offers` command.
        """
        return pulumi.get(self, "cloud_init")

    @cloud_init.setter
    def cloud_init(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "cloud_init", value)

    @_builtins.property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        A description for the server.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "description", value)

    @_builtins.property
    @pulumi.getter
    def domain(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The domain of the server.
        """
        return pulumi.get(self, "domain")

    @domain.setter
    def domain(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "domain", value)

    @_builtins.property
    @pulumi.getter
    def hostname(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The hostname of the server.
        """
        return pulumi.get(self, "hostname")

    @hostname.setter
    def hostname(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "hostname", value)

    @_builtins.property
    @pulumi.getter(name="installConfigAfterward")
    def install_config_afterward(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        If True, this boolean allows to create a server without the install config if you want to provide it later.
        """
        return pulumi.get(self, "install_config_afterward")

    @install_config_afterward.setter
    def install_config_afterward(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "install_config_afterward", value)

    @_builtins.property
    @pulumi.getter
    def ips(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ServerIpArgs']]]]:
        """
        (List of) The IPs of the server.
        """
        return pulumi.get(self, "ips")

    @ips.setter
    def ips(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ServerIpArgs']]]]):
        pulumi.set(self, "ips", value)

    @_builtins.property
    @pulumi.getter
    def ipv4s(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ServerIpv4Args']]]]:
        """
        (List of) The IPv4 addresses of the server.
        """
        return pulumi.get(self, "ipv4s")

    @ipv4s.setter
    def ipv4s(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ServerIpv4Args']]]]):
        pulumi.set(self, "ipv4s", value)

    @_builtins.property
    @pulumi.getter
    def ipv6s(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ServerIpv6Args']]]]:
        """
        (List of) The IPv6 addresses of the server.
        """
        return pulumi.get(self, "ipv6s")

    @ipv6s.setter
    def ipv6s(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ServerIpv6Args']]]]):
        pulumi.set(self, "ipv6s", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the server.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter
    def offer(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The offer UUID of the baremetal server.
        Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-servers-get-a-specific-elastic-metal-server) to find the right offer.

        > **Important:** Updates to `offer` will recreate the server.
        > **Important**  If you migrate to a monthly subscription, you cannot downgrade back to an hourly plan. Ensure that the monthly plan meets your needs before making the switch.
        """
        return pulumi.get(self, "offer")

    @offer.setter
    def offer(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "offer", value)

    @_builtins.property
    @pulumi.getter(name="offerId")
    def offer_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The ID of the offer.
        """
        return pulumi.get(self, "offer_id")

    @offer_id.setter
    def offer_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "offer_id", value)

    @_builtins.property
    @pulumi.getter(name="offerName")
    def offer_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the offer.
        """
        return pulumi.get(self, "offer_name")

    @offer_name.setter
    def offer_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "offer_name", value)

    @_builtins.property
    @pulumi.getter
    def options(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ServerOptionArgs']]]]:
        """
        The options to enable on the server.
        > The `options` block supports:
        """
        return pulumi.get(self, "options")

    @options.setter
    def options(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ServerOptionArgs']]]]):
        pulumi.set(self, "options", value)

    @_builtins.property
    @pulumi.getter(name="organizationId")
    def organization_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The organization ID the server is associated with.
        """
        return pulumi.get(self, "organization_id")

    @organization_id.setter
    def organization_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "organization_id", value)

    @_builtins.property
    @pulumi.getter
    def os(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The UUID of the os to install on the server.
        Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-os-list-available-oses) to find the right OS ID.
        > **Important:** Updates to `os` will reinstall the server.
        """
        return pulumi.get(self, "os")

    @os.setter
    def os(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "os", value)

    @_builtins.property
    @pulumi.getter(name="osName")
    def os_name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of the os.
        """
        return pulumi.get(self, "os_name")

    @os_name.setter
    def os_name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "os_name", value)

    @_builtins.property
    @pulumi.getter
    def partitioning(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The partitioning schema in JSON format
        """
        return pulumi.get(self, "partitioning")

    @partitioning.setter
    def partitioning(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "partitioning", value)

    @_builtins.property
    @pulumi.getter
    def password(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Password used for the installation. May be required depending on used os. Only one of `password` or `password_wo` should be specified.
        """
        return pulumi.get(self, "password")

    @password.setter
    def password(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "password", value)

    @_builtins.property
    @pulumi.getter(name="passwordWo")
    def password_wo(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        """
        return pulumi.get(self, "password_wo")

    @password_wo.setter
    def password_wo(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "password_wo", value)

    @_builtins.property
    @pulumi.getter(name="passwordWoVersion")
    def password_wo_version(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The version of the write-only password. To update the `password_wo`, you must also update the `password_wo_version`.
        """
        return pulumi.get(self, "password_wo_version")

    @password_wo_version.setter
    def password_wo_version(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "password_wo_version", value)

    @_builtins.property
    @pulumi.getter(name="privateIps")
    def private_ips(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateIpArgs']]]]:
        """
        The list of private IPv4 and IPv6 addresses associated with the resource.
        """
        return pulumi.get(self, "private_ips")

    @private_ips.setter
    def private_ips(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateIpArgs']]]]):
        pulumi.set(self, "private_ips", value)

    @_builtins.property
    @pulumi.getter(name="privateNetworks")
    def private_networks(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateNetworkArgs']]]]:
        """
        The private networks to attach to the server. For more information, see [the documentation](https://www.scaleway.com/en/docs/compute/elastic-metal/how-to/use-private-networks/)
        """
        return pulumi.get(self, "private_networks")

    @private_networks.setter
    def private_networks(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ServerPrivateNetworkArgs']]]]):
        pulumi.set(self, "private_networks", value)

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `project_id`) The ID of the project the server is associated with.
        """
        return pulumi.get(self, "project_id")

    @project_id.setter
    def project_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "project_id", value)

    @_builtins.property
    @pulumi.getter
    def protected(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Set to true to activate server protection option.
        """
        return pulumi.get(self, "protected")

    @protected.setter
    def protected(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "protected", value)

    @_builtins.property
    @pulumi.getter(name="reinstallOnConfigChanges")
    def reinstall_on_config_changes(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        If True, this boolean allows to reinstall the server on install config changes.
        > **Important:** Updates to `ssh_key_ids`, `user`, `password`, `service_user` or `service_password` will not take effect on the server, it requires to reinstall it. To do so please set 'reinstall_on_config_changes' argument to true.
        """
        return pulumi.get(self, "reinstall_on_config_changes")

    @reinstall_on_config_changes.setter
    def reinstall_on_config_changes(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "reinstall_on_config_changes", value)

    @_builtins.property
    @pulumi.getter(name="servicePassword")
    def service_password(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Password used for the service to install. May be required depending on used os. Only one of `service_password` or `service_password_wo` should be specified.
        """
        return pulumi.get(self, "service_password")

    @service_password.setter
    def service_password(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "service_password", value)

    @_builtins.property
    @pulumi.getter(name="servicePasswordWo")
    def service_password_wo(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        """
        return pulumi.get(self, "service_password_wo")

    @service_password_wo.setter
    def service_password_wo(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "service_password_wo", value)

    @_builtins.property
    @pulumi.getter(name="servicePasswordWoVersion")
    def service_password_wo_version(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The version of the write-only service password. To update the `service_password_wo`, you must also update the `service_password_wo_version`.
        """
        return pulumi.get(self, "service_password_wo_version")

    @service_password_wo_version.setter
    def service_password_wo_version(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "service_password_wo_version", value)

    @_builtins.property
    @pulumi.getter(name="serviceUser")
    def service_user(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        User used for the service to install.
        """
        return pulumi.get(self, "service_user")

    @service_user.setter
    def service_user(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "service_user", value)

    @_builtins.property
    @pulumi.getter(name="sshKeyIds")
    def ssh_key_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        List of SSH keys allowed to connect to the server.
        """
        return pulumi.get(self, "ssh_key_ids")

    @ssh_key_ids.setter
    def ssh_key_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "ssh_key_ids", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The tags associated with the server.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter
    def user(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        User used for the installation.
        """
        return pulumi.get(self, "user")

    @user.setter
    def user(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "user", value)

    @_builtins.property
    @pulumi.getter
    def zone(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `zone`) The zone in which the server should be created.
        """
        return pulumi.get(self, "zone")

    @zone.setter
    def zone(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "zone", value)


@pulumi.type_token("scaleway:elasticmetal/server:Server")
class Server(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cloud_init: Optional[pulumi.Input[_builtins.str]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 hostname: Optional[pulumi.Input[_builtins.str]] = None,
                 install_config_afterward: Optional[pulumi.Input[_builtins.bool]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 offer: Optional[pulumi.Input[_builtins.str]] = None,
                 options: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerOptionArgs', 'ServerOptionArgsDict']]]]] = None,
                 os: Optional[pulumi.Input[_builtins.str]] = None,
                 partitioning: Optional[pulumi.Input[_builtins.str]] = None,
                 password: Optional[pulumi.Input[_builtins.str]] = None,
                 password_wo: Optional[pulumi.Input[_builtins.str]] = None,
                 password_wo_version: Optional[pulumi.Input[_builtins.int]] = None,
                 private_ips: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerPrivateIpArgs', 'ServerPrivateIpArgsDict']]]]] = None,
                 private_networks: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerPrivateNetworkArgs', 'ServerPrivateNetworkArgsDict']]]]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 protected: Optional[pulumi.Input[_builtins.bool]] = None,
                 reinstall_on_config_changes: Optional[pulumi.Input[_builtins.bool]] = None,
                 service_password: Optional[pulumi.Input[_builtins.str]] = None,
                 service_password_wo: Optional[pulumi.Input[_builtins.str]] = None,
                 service_password_wo_version: Optional[pulumi.Input[_builtins.int]] = None,
                 service_user: Optional[pulumi.Input[_builtins.str]] = None,
                 ssh_key_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 user: Optional[pulumi.Input[_builtins.str]] = None,
                 zone: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_scaleway as scaleway
        import pulumi_std as std
        import pulumiverse_scaleway as scaleway

        ### With cloud-init
        my_ssh_key = scaleway.iam.get_ssh_key(name="main")
        my_os = scaleway.elasticmetal.get_os(zone="fr-par-1",
            name="Ubuntu",
            version="22.04 LTS (Jammy Jellyfish)")
        my_offer = scaleway.elasticmetal.get_offer(zone="fr-par-2",
            name="EM-I220E-NVME")
        my_server_ci = scaleway.elasticmetal.Server("my_server_ci",
            zone="fr-par-2",
            offer=my_offer.offer_id,
            os=my_os.os_id,
            ssh_key_ids=[my_ssh_key.id],
            cloud_init=std.index.file(input="userdata.yaml")["result"])
        ```

        ## Import

        Baremetal servers can be imported using the `{zone}/{id}`, e.g.

        bash

        ```sh
        $ pulumi import scaleway:elasticmetal/server:Server web fr-par-2/11111111-1111-1111-1111-111111111111
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] cloud_init: Configuration data to pass to cloud-init such as a YAML cloud config or a user-data script. Accepts either a string containing the content or a path to a file (for example `file("cloud-init.yml")`). Max length: 127998 characters. Updates to `cloud_init` will update the server user-data via the API and do not trigger a reinstall; however, a reboot of the server is required for the OS to re-run cloud-init and apply the changes. Only supported for Offers that have cloud-init enabled. You can check available offers with `scw baremetal list offers` command.
        :param pulumi.Input[_builtins.str] description: A description for the server.
        :param pulumi.Input[_builtins.str] hostname: The hostname of the server.
        :param pulumi.Input[_builtins.bool] install_config_afterward: If True, this boolean allows to create a server without the install config if you want to provide it later.
        :param pulumi.Input[_builtins.str] name: The name of the server.
        :param pulumi.Input[_builtins.str] offer: The offer UUID of the baremetal server.
               Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-servers-get-a-specific-elastic-metal-server) to find the right offer.
               
               > **Important:** Updates to `offer` will recreate the server.
               > **Important**  If you migrate to a monthly subscription, you cannot downgrade back to an hourly plan. Ensure that the monthly plan meets your needs before making the switch.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ServerOptionArgs', 'ServerOptionArgsDict']]]] options: The options to enable on the server.
               > The `options` block supports:
        :param pulumi.Input[_builtins.str] os: The UUID of the os to install on the server.
               Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-os-list-available-oses) to find the right OS ID.
               > **Important:** Updates to `os` will reinstall the server.
        :param pulumi.Input[_builtins.str] partitioning: The partitioning schema in JSON format
        :param pulumi.Input[_builtins.str] password: Password used for the installation. May be required depending on used os. Only one of `password` or `password_wo` should be specified.
        :param pulumi.Input[_builtins.str] password_wo: **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        :param pulumi.Input[_builtins.int] password_wo_version: The version of the write-only password. To update the `password_wo`, you must also update the `password_wo_version`.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ServerPrivateIpArgs', 'ServerPrivateIpArgsDict']]]] private_ips: The list of private IPv4 and IPv6 addresses associated with the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ServerPrivateNetworkArgs', 'ServerPrivateNetworkArgsDict']]]] private_networks: The private networks to attach to the server. For more information, see [the documentation](https://www.scaleway.com/en/docs/compute/elastic-metal/how-to/use-private-networks/)
        :param pulumi.Input[_builtins.str] project_id: `project_id`) The ID of the project the server is associated with.
        :param pulumi.Input[_builtins.bool] protected: Set to true to activate server protection option.
        :param pulumi.Input[_builtins.bool] reinstall_on_config_changes: If True, this boolean allows to reinstall the server on install config changes.
               > **Important:** Updates to `ssh_key_ids`, `user`, `password`, `service_user` or `service_password` will not take effect on the server, it requires to reinstall it. To do so please set 'reinstall_on_config_changes' argument to true.
        :param pulumi.Input[_builtins.str] service_password: Password used for the service to install. May be required depending on used os. Only one of `service_password` or `service_password_wo` should be specified.
        :param pulumi.Input[_builtins.str] service_password_wo: **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        :param pulumi.Input[_builtins.int] service_password_wo_version: The version of the write-only service password. To update the `service_password_wo`, you must also update the `service_password_wo_version`.
        :param pulumi.Input[_builtins.str] service_user: User used for the service to install.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] ssh_key_ids: List of SSH keys allowed to connect to the server.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags associated with the server.
        :param pulumi.Input[_builtins.str] user: User used for the installation.
        :param pulumi.Input[_builtins.str] zone: `zone`) The zone in which the server should be created.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ServerArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_scaleway as scaleway
        import pulumi_std as std
        import pulumiverse_scaleway as scaleway

        ### With cloud-init
        my_ssh_key = scaleway.iam.get_ssh_key(name="main")
        my_os = scaleway.elasticmetal.get_os(zone="fr-par-1",
            name="Ubuntu",
            version="22.04 LTS (Jammy Jellyfish)")
        my_offer = scaleway.elasticmetal.get_offer(zone="fr-par-2",
            name="EM-I220E-NVME")
        my_server_ci = scaleway.elasticmetal.Server("my_server_ci",
            zone="fr-par-2",
            offer=my_offer.offer_id,
            os=my_os.os_id,
            ssh_key_ids=[my_ssh_key.id],
            cloud_init=std.index.file(input="userdata.yaml")["result"])
        ```

        ## Import

        Baremetal servers can be imported using the `{zone}/{id}`, e.g.

        bash

        ```sh
        $ pulumi import scaleway:elasticmetal/server:Server web fr-par-2/11111111-1111-1111-1111-111111111111
        ```

        :param str resource_name: The name of the resource.
        :param ServerArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ServerArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cloud_init: Optional[pulumi.Input[_builtins.str]] = None,
                 description: Optional[pulumi.Input[_builtins.str]] = None,
                 hostname: Optional[pulumi.Input[_builtins.str]] = None,
                 install_config_afterward: Optional[pulumi.Input[_builtins.bool]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 offer: Optional[pulumi.Input[_builtins.str]] = None,
                 options: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerOptionArgs', 'ServerOptionArgsDict']]]]] = None,
                 os: Optional[pulumi.Input[_builtins.str]] = None,
                 partitioning: Optional[pulumi.Input[_builtins.str]] = None,
                 password: Optional[pulumi.Input[_builtins.str]] = None,
                 password_wo: Optional[pulumi.Input[_builtins.str]] = None,
                 password_wo_version: Optional[pulumi.Input[_builtins.int]] = None,
                 private_ips: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerPrivateIpArgs', 'ServerPrivateIpArgsDict']]]]] = None,
                 private_networks: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerPrivateNetworkArgs', 'ServerPrivateNetworkArgsDict']]]]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 protected: Optional[pulumi.Input[_builtins.bool]] = None,
                 reinstall_on_config_changes: Optional[pulumi.Input[_builtins.bool]] = None,
                 service_password: Optional[pulumi.Input[_builtins.str]] = None,
                 service_password_wo: Optional[pulumi.Input[_builtins.str]] = None,
                 service_password_wo_version: Optional[pulumi.Input[_builtins.int]] = None,
                 service_user: Optional[pulumi.Input[_builtins.str]] = None,
                 ssh_key_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 user: Optional[pulumi.Input[_builtins.str]] = None,
                 zone: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ServerArgs.__new__(ServerArgs)

            __props__.__dict__["cloud_init"] = cloud_init
            __props__.__dict__["description"] = description
            __props__.__dict__["hostname"] = hostname
            __props__.__dict__["install_config_afterward"] = install_config_afterward
            __props__.__dict__["name"] = name
            if offer is None and not opts.urn:
                raise TypeError("Missing required property 'offer'")
            __props__.__dict__["offer"] = offer
            __props__.__dict__["options"] = options
            __props__.__dict__["os"] = os
            __props__.__dict__["partitioning"] = partitioning
            __props__.__dict__["password"] = None if password is None else pulumi.Output.secret(password)
            __props__.__dict__["password_wo"] = None if password_wo is None else pulumi.Output.secret(password_wo)
            __props__.__dict__["password_wo_version"] = password_wo_version
            __props__.__dict__["private_ips"] = private_ips
            __props__.__dict__["private_networks"] = private_networks
            __props__.__dict__["project_id"] = project_id
            __props__.__dict__["protected"] = protected
            __props__.__dict__["reinstall_on_config_changes"] = reinstall_on_config_changes
            __props__.__dict__["service_password"] = None if service_password is None else pulumi.Output.secret(service_password)
            __props__.__dict__["service_password_wo"] = None if service_password_wo is None else pulumi.Output.secret(service_password_wo)
            __props__.__dict__["service_password_wo_version"] = service_password_wo_version
            __props__.__dict__["service_user"] = service_user
            __props__.__dict__["ssh_key_ids"] = ssh_key_ids
            __props__.__dict__["tags"] = tags
            __props__.__dict__["user"] = user
            __props__.__dict__["zone"] = zone
            __props__.__dict__["domain"] = None
            __props__.__dict__["ips"] = None
            __props__.__dict__["ipv4s"] = None
            __props__.__dict__["ipv6s"] = None
            __props__.__dict__["offer_id"] = None
            __props__.__dict__["offer_name"] = None
            __props__.__dict__["organization_id"] = None
            __props__.__dict__["os_name"] = None
        alias_opts = pulumi.ResourceOptions(aliases=[pulumi.Alias(type_="scaleway:index/baremetalServer:BaremetalServer")])
        opts = pulumi.ResourceOptions.merge(opts, alias_opts)
        secret_opts = pulumi.ResourceOptions(additional_secret_outputs=["password", "passwordWo", "servicePassword", "servicePasswordWo"])
        opts = pulumi.ResourceOptions.merge(opts, secret_opts)
        super(Server, __self__).__init__(
            'scaleway:elasticmetal/server:Server',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            cloud_init: Optional[pulumi.Input[_builtins.str]] = None,
            description: Optional[pulumi.Input[_builtins.str]] = None,
            domain: Optional[pulumi.Input[_builtins.str]] = None,
            hostname: Optional[pulumi.Input[_builtins.str]] = None,
            install_config_afterward: Optional[pulumi.Input[_builtins.bool]] = None,
            ips: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerIpArgs', 'ServerIpArgsDict']]]]] = None,
            ipv4s: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerIpv4Args', 'ServerIpv4ArgsDict']]]]] = None,
            ipv6s: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerIpv6Args', 'ServerIpv6ArgsDict']]]]] = None,
            name: Optional[pulumi.Input[_builtins.str]] = None,
            offer: Optional[pulumi.Input[_builtins.str]] = None,
            offer_id: Optional[pulumi.Input[_builtins.str]] = None,
            offer_name: Optional[pulumi.Input[_builtins.str]] = None,
            options: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerOptionArgs', 'ServerOptionArgsDict']]]]] = None,
            organization_id: Optional[pulumi.Input[_builtins.str]] = None,
            os: Optional[pulumi.Input[_builtins.str]] = None,
            os_name: Optional[pulumi.Input[_builtins.str]] = None,
            partitioning: Optional[pulumi.Input[_builtins.str]] = None,
            password: Optional[pulumi.Input[_builtins.str]] = None,
            password_wo: Optional[pulumi.Input[_builtins.str]] = None,
            password_wo_version: Optional[pulumi.Input[_builtins.int]] = None,
            private_ips: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerPrivateIpArgs', 'ServerPrivateIpArgsDict']]]]] = None,
            private_networks: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ServerPrivateNetworkArgs', 'ServerPrivateNetworkArgsDict']]]]] = None,
            project_id: Optional[pulumi.Input[_builtins.str]] = None,
            protected: Optional[pulumi.Input[_builtins.bool]] = None,
            reinstall_on_config_changes: Optional[pulumi.Input[_builtins.bool]] = None,
            service_password: Optional[pulumi.Input[_builtins.str]] = None,
            service_password_wo: Optional[pulumi.Input[_builtins.str]] = None,
            service_password_wo_version: Optional[pulumi.Input[_builtins.int]] = None,
            service_user: Optional[pulumi.Input[_builtins.str]] = None,
            ssh_key_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            user: Optional[pulumi.Input[_builtins.str]] = None,
            zone: Optional[pulumi.Input[_builtins.str]] = None) -> 'Server':
        """
        Get an existing Server resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] cloud_init: Configuration data to pass to cloud-init such as a YAML cloud config or a user-data script. Accepts either a string containing the content or a path to a file (for example `file("cloud-init.yml")`). Max length: 127998 characters. Updates to `cloud_init` will update the server user-data via the API and do not trigger a reinstall; however, a reboot of the server is required for the OS to re-run cloud-init and apply the changes. Only supported for Offers that have cloud-init enabled. You can check available offers with `scw baremetal list offers` command.
        :param pulumi.Input[_builtins.str] description: A description for the server.
        :param pulumi.Input[_builtins.str] domain: The domain of the server.
        :param pulumi.Input[_builtins.str] hostname: The hostname of the server.
        :param pulumi.Input[_builtins.bool] install_config_afterward: If True, this boolean allows to create a server without the install config if you want to provide it later.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ServerIpArgs', 'ServerIpArgsDict']]]] ips: (List of) The IPs of the server.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ServerIpv4Args', 'ServerIpv4ArgsDict']]]] ipv4s: (List of) The IPv4 addresses of the server.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ServerIpv6Args', 'ServerIpv6ArgsDict']]]] ipv6s: (List of) The IPv6 addresses of the server.
        :param pulumi.Input[_builtins.str] name: The name of the server.
        :param pulumi.Input[_builtins.str] offer: The offer UUID of the baremetal server.
               Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-servers-get-a-specific-elastic-metal-server) to find the right offer.
               
               > **Important:** Updates to `offer` will recreate the server.
               > **Important**  If you migrate to a monthly subscription, you cannot downgrade back to an hourly plan. Ensure that the monthly plan meets your needs before making the switch.
        :param pulumi.Input[_builtins.str] offer_id: The ID of the offer.
        :param pulumi.Input[_builtins.str] offer_name: The name of the offer.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ServerOptionArgs', 'ServerOptionArgsDict']]]] options: The options to enable on the server.
               > The `options` block supports:
        :param pulumi.Input[_builtins.str] organization_id: The organization ID the server is associated with.
        :param pulumi.Input[_builtins.str] os: The UUID of the os to install on the server.
               Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-os-list-available-oses) to find the right OS ID.
               > **Important:** Updates to `os` will reinstall the server.
        :param pulumi.Input[_builtins.str] os_name: The name of the os.
        :param pulumi.Input[_builtins.str] partitioning: The partitioning schema in JSON format
        :param pulumi.Input[_builtins.str] password: Password used for the installation. May be required depending on used os. Only one of `password` or `password_wo` should be specified.
        :param pulumi.Input[_builtins.str] password_wo: **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        :param pulumi.Input[_builtins.int] password_wo_version: The version of the write-only password. To update the `password_wo`, you must also update the `password_wo_version`.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ServerPrivateIpArgs', 'ServerPrivateIpArgsDict']]]] private_ips: The list of private IPv4 and IPv6 addresses associated with the resource.
        :param pulumi.Input[Sequence[pulumi.Input[Union['ServerPrivateNetworkArgs', 'ServerPrivateNetworkArgsDict']]]] private_networks: The private networks to attach to the server. For more information, see [the documentation](https://www.scaleway.com/en/docs/compute/elastic-metal/how-to/use-private-networks/)
        :param pulumi.Input[_builtins.str] project_id: `project_id`) The ID of the project the server is associated with.
        :param pulumi.Input[_builtins.bool] protected: Set to true to activate server protection option.
        :param pulumi.Input[_builtins.bool] reinstall_on_config_changes: If True, this boolean allows to reinstall the server on install config changes.
               > **Important:** Updates to `ssh_key_ids`, `user`, `password`, `service_user` or `service_password` will not take effect on the server, it requires to reinstall it. To do so please set 'reinstall_on_config_changes' argument to true.
        :param pulumi.Input[_builtins.str] service_password: Password used for the service to install. May be required depending on used os. Only one of `service_password` or `service_password_wo` should be specified.
        :param pulumi.Input[_builtins.str] service_password_wo: **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        :param pulumi.Input[_builtins.int] service_password_wo_version: The version of the write-only service password. To update the `service_password_wo`, you must also update the `service_password_wo_version`.
        :param pulumi.Input[_builtins.str] service_user: User used for the service to install.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] ssh_key_ids: List of SSH keys allowed to connect to the server.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags associated with the server.
        :param pulumi.Input[_builtins.str] user: User used for the installation.
        :param pulumi.Input[_builtins.str] zone: `zone`) The zone in which the server should be created.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ServerState.__new__(_ServerState)

        __props__.__dict__["cloud_init"] = cloud_init
        __props__.__dict__["description"] = description
        __props__.__dict__["domain"] = domain
        __props__.__dict__["hostname"] = hostname
        __props__.__dict__["install_config_afterward"] = install_config_afterward
        __props__.__dict__["ips"] = ips
        __props__.__dict__["ipv4s"] = ipv4s
        __props__.__dict__["ipv6s"] = ipv6s
        __props__.__dict__["name"] = name
        __props__.__dict__["offer"] = offer
        __props__.__dict__["offer_id"] = offer_id
        __props__.__dict__["offer_name"] = offer_name
        __props__.__dict__["options"] = options
        __props__.__dict__["organization_id"] = organization_id
        __props__.__dict__["os"] = os
        __props__.__dict__["os_name"] = os_name
        __props__.__dict__["partitioning"] = partitioning
        __props__.__dict__["password"] = password
        __props__.__dict__["password_wo"] = password_wo
        __props__.__dict__["password_wo_version"] = password_wo_version
        __props__.__dict__["private_ips"] = private_ips
        __props__.__dict__["private_networks"] = private_networks
        __props__.__dict__["project_id"] = project_id
        __props__.__dict__["protected"] = protected
        __props__.__dict__["reinstall_on_config_changes"] = reinstall_on_config_changes
        __props__.__dict__["service_password"] = service_password
        __props__.__dict__["service_password_wo"] = service_password_wo
        __props__.__dict__["service_password_wo_version"] = service_password_wo_version
        __props__.__dict__["service_user"] = service_user
        __props__.__dict__["ssh_key_ids"] = ssh_key_ids
        __props__.__dict__["tags"] = tags
        __props__.__dict__["user"] = user
        __props__.__dict__["zone"] = zone
        return Server(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="cloudInit")
    def cloud_init(self) -> pulumi.Output[_builtins.str]:
        """
        Configuration data to pass to cloud-init such as a YAML cloud config or a user-data script. Accepts either a string containing the content or a path to a file (for example `file("cloud-init.yml")`). Max length: 127998 characters. Updates to `cloud_init` will update the server user-data via the API and do not trigger a reinstall; however, a reboot of the server is required for the OS to re-run cloud-init and apply the changes. Only supported for Offers that have cloud-init enabled. You can check available offers with `scw baremetal list offers` command.
        """
        return pulumi.get(self, "cloud_init")

    @_builtins.property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        A description for the server.
        """
        return pulumi.get(self, "description")

    @_builtins.property
    @pulumi.getter
    def domain(self) -> pulumi.Output[_builtins.str]:
        """
        The domain of the server.
        """
        return pulumi.get(self, "domain")

    @_builtins.property
    @pulumi.getter
    def hostname(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The hostname of the server.
        """
        return pulumi.get(self, "hostname")

    @_builtins.property
    @pulumi.getter(name="installConfigAfterward")
    def install_config_afterward(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        If True, this boolean allows to create a server without the install config if you want to provide it later.
        """
        return pulumi.get(self, "install_config_afterward")

    @_builtins.property
    @pulumi.getter
    def ips(self) -> pulumi.Output[Sequence['outputs.ServerIp']]:
        """
        (List of) The IPs of the server.
        """
        return pulumi.get(self, "ips")

    @_builtins.property
    @pulumi.getter
    def ipv4s(self) -> pulumi.Output[Sequence['outputs.ServerIpv4']]:
        """
        (List of) The IPv4 addresses of the server.
        """
        return pulumi.get(self, "ipv4s")

    @_builtins.property
    @pulumi.getter
    def ipv6s(self) -> pulumi.Output[Sequence['outputs.ServerIpv6']]:
        """
        (List of) The IPv6 addresses of the server.
        """
        return pulumi.get(self, "ipv6s")

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Output[_builtins.str]:
        """
        The name of the server.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def offer(self) -> pulumi.Output[_builtins.str]:
        """
        The offer UUID of the baremetal server.
        Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-servers-get-a-specific-elastic-metal-server) to find the right offer.

        > **Important:** Updates to `offer` will recreate the server.
        > **Important**  If you migrate to a monthly subscription, you cannot downgrade back to an hourly plan. Ensure that the monthly plan meets your needs before making the switch.
        """
        return pulumi.get(self, "offer")

    @_builtins.property
    @pulumi.getter(name="offerId")
    def offer_id(self) -> pulumi.Output[_builtins.str]:
        """
        The ID of the offer.
        """
        return pulumi.get(self, "offer_id")

    @_builtins.property
    @pulumi.getter(name="offerName")
    def offer_name(self) -> pulumi.Output[_builtins.str]:
        """
        The name of the offer.
        """
        return pulumi.get(self, "offer_name")

    @_builtins.property
    @pulumi.getter
    def options(self) -> pulumi.Output[Optional[Sequence['outputs.ServerOption']]]:
        """
        The options to enable on the server.
        > The `options` block supports:
        """
        return pulumi.get(self, "options")

    @_builtins.property
    @pulumi.getter(name="organizationId")
    def organization_id(self) -> pulumi.Output[_builtins.str]:
        """
        The organization ID the server is associated with.
        """
        return pulumi.get(self, "organization_id")

    @_builtins.property
    @pulumi.getter
    def os(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The UUID of the os to install on the server.
        Use [this endpoint](https://www.scaleway.com/en/developers/api/elastic-metal/#path-os-list-available-oses) to find the right OS ID.
        > **Important:** Updates to `os` will reinstall the server.
        """
        return pulumi.get(self, "os")

    @_builtins.property
    @pulumi.getter(name="osName")
    def os_name(self) -> pulumi.Output[_builtins.str]:
        """
        The name of the os.
        """
        return pulumi.get(self, "os_name")

    @_builtins.property
    @pulumi.getter
    def partitioning(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The partitioning schema in JSON format
        """
        return pulumi.get(self, "partitioning")

    @_builtins.property
    @pulumi.getter
    def password(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        Password used for the installation. May be required depending on used os. Only one of `password` or `password_wo` should be specified.
        """
        return pulumi.get(self, "password")

    @_builtins.property
    @pulumi.getter(name="passwordWo")
    def password_wo(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        """
        return pulumi.get(self, "password_wo")

    @_builtins.property
    @pulumi.getter(name="passwordWoVersion")
    def password_wo_version(self) -> pulumi.Output[Optional[_builtins.int]]:
        """
        The version of the write-only password. To update the `password_wo`, you must also update the `password_wo_version`.
        """
        return pulumi.get(self, "password_wo_version")

    @_builtins.property
    @pulumi.getter(name="privateIps")
    def private_ips(self) -> pulumi.Output[Sequence['outputs.ServerPrivateIp']]:
        """
        The list of private IPv4 and IPv6 addresses associated with the resource.
        """
        return pulumi.get(self, "private_ips")

    @_builtins.property
    @pulumi.getter(name="privateNetworks")
    def private_networks(self) -> pulumi.Output[Optional[Sequence['outputs.ServerPrivateNetwork']]]:
        """
        The private networks to attach to the server. For more information, see [the documentation](https://www.scaleway.com/en/docs/compute/elastic-metal/how-to/use-private-networks/)
        """
        return pulumi.get(self, "private_networks")

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> pulumi.Output[_builtins.str]:
        """
        `project_id`) The ID of the project the server is associated with.
        """
        return pulumi.get(self, "project_id")

    @_builtins.property
    @pulumi.getter
    def protected(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        Set to true to activate server protection option.
        """
        return pulumi.get(self, "protected")

    @_builtins.property
    @pulumi.getter(name="reinstallOnConfigChanges")
    def reinstall_on_config_changes(self) -> pulumi.Output[Optional[_builtins.bool]]:
        """
        If True, this boolean allows to reinstall the server on install config changes.
        > **Important:** Updates to `ssh_key_ids`, `user`, `password`, `service_user` or `service_password` will not take effect on the server, it requires to reinstall it. To do so please set 'reinstall_on_config_changes' argument to true.
        """
        return pulumi.get(self, "reinstall_on_config_changes")

    @_builtins.property
    @pulumi.getter(name="servicePassword")
    def service_password(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        Password used for the service to install. May be required depending on used os. Only one of `service_password` or `service_password_wo` should be specified.
        """
        return pulumi.get(self, "service_password")

    @_builtins.property
    @pulumi.getter(name="servicePasswordWo")
    def service_password_wo(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        **NOTE:** This field is write-only and its value will not be updated in state as part of read operations.
        """
        return pulumi.get(self, "service_password_wo")

    @_builtins.property
    @pulumi.getter(name="servicePasswordWoVersion")
    def service_password_wo_version(self) -> pulumi.Output[Optional[_builtins.int]]:
        """
        The version of the write-only service password. To update the `service_password_wo`, you must also update the `service_password_wo_version`.
        """
        return pulumi.get(self, "service_password_wo_version")

    @_builtins.property
    @pulumi.getter(name="serviceUser")
    def service_user(self) -> pulumi.Output[_builtins.str]:
        """
        User used for the service to install.
        """
        return pulumi.get(self, "service_user")

    @_builtins.property
    @pulumi.getter(name="sshKeyIds")
    def ssh_key_ids(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        List of SSH keys allowed to connect to the server.
        """
        return pulumi.get(self, "ssh_key_ids")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Sequence[_builtins.str]]:
        """
        The tags associated with the server.
        """
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter
    def user(self) -> pulumi.Output[_builtins.str]:
        """
        User used for the installation.
        """
        return pulumi.get(self, "user")

    @_builtins.property
    @pulumi.getter
    def zone(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        `zone`) The zone in which the server should be created.
        """
        return pulumi.get(self, "zone")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'HostingCpanelUrlArgs',
    'HostingCpanelUrlArgsDict',
    'HostingNameServerArgs',
    'HostingNameServerArgsDict',
    'HostingOptionArgs',
    'HostingOptionArgsDict',
    'HostingRecordArgs',
    'HostingRecordArgsDict',
]

MYPY = False

if not MYPY:
    class HostingCpanelUrlArgsDict(TypedDict):
        dashboard: NotRequired[pulumi.Input[_builtins.str]]
        """
        The URL of the Dashboard.
        """
        webmail: NotRequired[pulumi.Input[_builtins.str]]
        """
        The URL of the Webmail interface.
        """
elif False:
    HostingCpanelUrlArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class HostingCpanelUrlArgs:
    def __init__(__self__, *,
                 dashboard: Optional[pulumi.Input[_builtins.str]] = None,
                 webmail: Optional[pulumi.Input[_builtins.str]] = None):
        """
        :param pulumi.Input[_builtins.str] dashboard: The URL of the Dashboard.
        :param pulumi.Input[_builtins.str] webmail: The URL of the Webmail interface.
        """
        if dashboard is not None:
            pulumi.set(__self__, "dashboard", dashboard)
        if webmail is not None:
            pulumi.set(__self__, "webmail", webmail)

    @_builtins.property
    @pulumi.getter
    def dashboard(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The URL of the Dashboard.
        """
        return pulumi.get(self, "dashboard")

    @dashboard.setter
    def dashboard(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "dashboard", value)

    @_builtins.property
    @pulumi.getter
    def webmail(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The URL of the Webmail interface.
        """
        return pulumi.get(self, "webmail")

    @webmail.setter
    def webmail(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "webmail", value)


if not MYPY:
    class HostingNameServerArgsDict(TypedDict):
        hostname: NotRequired[pulumi.Input[_builtins.str]]
        """
        Hostname of the server
        """
        is_default: NotRequired[pulumi.Input[_builtins.bool]]
        """
        Whether or not the webhosting is the default one
        """
        status: NotRequired[pulumi.Input[_builtins.str]]
        """
        The hosting status.
        """
elif False:
    HostingNameServerArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class HostingNameServerArgs:
    def __init__(__self__, *,
                 hostname: Optional[pulumi.Input[_builtins.str]] = None,
                 is_default: Optional[pulumi.Input[_builtins.bool]] = None,
                 status: Optional[pulumi.Input[_builtins.str]] = None):
        """
        :param pulumi.Input[_builtins.str] hostname: Hostname of the server
        :param pulumi.Input[_builtins.bool] is_default: Whether or not the webhosting is the default one
        :param pulumi.Input[_builtins.str] status: The hosting status.
        """
        if hostname is not None:
            pulumi.set(__self__, "hostname", hostname)
        if is_default is not None:
            pulumi.set(__self__, "is_default", is_default)
        if status is not None:
            pulumi.set(__self__, "status", status)

    @_builtins.property
    @pulumi.getter
    def hostname(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Hostname of the server
        """
        return pulumi.get(self, "hostname")

    @hostname.setter
    def hostname(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "hostname", value)

    @_builtins.property
    @pulumi.getter(name="isDefault")
    def is_default(self) -> Optional[pulumi.Input[_builtins.bool]]:
        """
        Whether or not the webhosting is the default one
        """
        return pulumi.get(self, "is_default")

    @is_default.setter
    def is_default(self, value: Optional[pulumi.Input[_builtins.bool]]):
        pulumi.set(self, "is_default", value)

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The hosting status.
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "status", value)


if not MYPY:
    class HostingOptionArgsDict(TypedDict):
        id: NotRequired[pulumi.Input[_builtins.str]]
        """
        The option ID.
        """
        name: NotRequired[pulumi.Input[_builtins.str]]
        """
        The option name.
        """
elif False:
    HostingOptionArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class HostingOptionArgs:
    def __init__(__self__, *,
                 id: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None):
        """
        :param pulumi.Input[_builtins.str] id: The option ID.
        :param pulumi.Input[_builtins.str] name: The option name.
        """
        if id is not None:
            pulumi.set(__self__, "id", id)
        if name is not None:
            pulumi.set(__self__, "name", name)

    @_builtins.property
    @pulumi.getter
    def id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The option ID.
        """
        return pulumi.get(self, "id")

    @id.setter
    def id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "id", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The option name.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)


if not MYPY:
    class HostingRecordArgsDict(TypedDict):
        name: NotRequired[pulumi.Input[_builtins.str]]
        """
        The option name.
        """
        priority: NotRequired[pulumi.Input[_builtins.int]]
        """
        Priority of DNS records associated with the webhosting.
        """
        status: NotRequired[pulumi.Input[_builtins.str]]
        """
        The hosting status.
        """
        ttl: NotRequired[pulumi.Input[_builtins.int]]
        """
        Time to live in seconds of the record
        """
        type: NotRequired[pulumi.Input[_builtins.str]]
        """
        Type of the DNS record
        """
        value: NotRequired[pulumi.Input[_builtins.str]]
        """
        Value of the DNS record
        """
elif False:
    HostingRecordArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class HostingRecordArgs:
    def __init__(__self__, *,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 priority: Optional[pulumi.Input[_builtins.int]] = None,
                 status: Optional[pulumi.Input[_builtins.str]] = None,
                 ttl: Optional[pulumi.Input[_builtins.int]] = None,
                 type: Optional[pulumi.Input[_builtins.str]] = None,
                 value: Optional[pulumi.Input[_builtins.str]] = None):
        """
        :param pulumi.Input[_builtins.str] name: The option name.
        :param pulumi.Input[_builtins.int] priority: Priority of DNS records associated with the webhosting.
        :param pulumi.Input[_builtins.str] status: The hosting status.
        :param pulumi.Input[_builtins.int] ttl: Time to live in seconds of the record
        :param pulumi.Input[_builtins.str] type: Type of the DNS record
        :param pulumi.Input[_builtins.str] value: Value of the DNS record
        """
        if name is not None:
            pulumi.set(__self__, "name", name)
        if priority is not None:
            pulumi.set(__self__, "priority", priority)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if ttl is not None:
            pulumi.set(__self__, "ttl", ttl)
        if type is not None:
            pulumi.set(__self__, "type", type)
        if value is not None:
            pulumi.set(__self__, "value", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The option name.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter
    def priority(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Priority of DNS records associated with the webhosting.
        """
        return pulumi.get(self, "priority")

    @priority.setter
    def priority(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "priority", value)

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The hosting status.
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "status", value)

    @_builtins.property
    @pulumi.getter
    def ttl(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        Time to live in seconds of the record
        """
        return pulumi.get(self, "ttl")

    @ttl.setter
    def ttl(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "ttl", value)

    @_builtins.property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Type of the DNS record
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "type", value)

    @_builtins.property
    @pulumi.getter
    def value(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Value of the DNS record
        """
        return pulumi.get(self, "value")

    @value.setter
    def value(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "value", value)





# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['InstanceTemplateArgs', 'InstanceTemplate']

@pulumi.input_type
class InstanceTemplateArgs:
    def __init__(__self__, *,
                 commercial_type: pulumi.Input[_builtins.str],
                 cloud_init: Optional[pulumi.Input[_builtins.str]] = None,
                 image_id: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 placement_group_id: Optional[pulumi.Input[_builtins.str]] = None,
                 private_network_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 public_ips_v4_count: Optional[pulumi.Input[_builtins.int]] = None,
                 public_ips_v6_count: Optional[pulumi.Input[_builtins.int]] = None,
                 security_group_id: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 volumes: Optional[pulumi.Input[Sequence[pulumi.Input['InstanceTemplateVolumeArgs']]]] = None,
                 zone: Optional[pulumi.Input[_builtins.str]] = None):
        """
        The set of arguments for constructing a InstanceTemplate resource.
        :param pulumi.Input[_builtins.str] commercial_type: The name of Instance commercial type.
        :param pulumi.Input[_builtins.str] cloud_init: The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        :param pulumi.Input[_builtins.str] image_id: The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        :param pulumi.Input[_builtins.str] name: The Instance group template.
        :param pulumi.Input[_builtins.str] placement_group_id: The instance placement group ID. This is optional, but it is highly recommended to set a preference for Instance location within Availability Zone.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] private_network_ids: The private Network IDs to attach to the new Instance.
        :param pulumi.Input[_builtins.str] project_id: `project_id`) The ID of the Project the Instance template is associated with.
        :param pulumi.Input[_builtins.int] public_ips_v4_count: The number of flexible IPv4 addresses to attach to the new Instance.
        :param pulumi.Input[_builtins.int] public_ips_v6_count: The number of flexible IPv6 addresses to attach to the new Instance.
        :param pulumi.Input[_builtins.str] security_group_id: The instance security group ID.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags associated with the Instance template.
        :param pulumi.Input[Sequence[pulumi.Input['InstanceTemplateVolumeArgs']]] volumes: The template of Instance volume.
        :param pulumi.Input[_builtins.str] zone: `zone`) The zone in which the Instance template exists.
        """
        pulumi.set(__self__, "commercial_type", commercial_type)
        if cloud_init is not None:
            pulumi.set(__self__, "cloud_init", cloud_init)
        if image_id is not None:
            pulumi.set(__self__, "image_id", image_id)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if placement_group_id is not None:
            pulumi.set(__self__, "placement_group_id", placement_group_id)
        if private_network_ids is not None:
            pulumi.set(__self__, "private_network_ids", private_network_ids)
        if project_id is not None:
            pulumi.set(__self__, "project_id", project_id)
        if public_ips_v4_count is not None:
            pulumi.set(__self__, "public_ips_v4_count", public_ips_v4_count)
        if public_ips_v6_count is not None:
            pulumi.set(__self__, "public_ips_v6_count", public_ips_v6_count)
        if security_group_id is not None:
            pulumi.set(__self__, "security_group_id", security_group_id)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if volumes is not None:
            pulumi.set(__self__, "volumes", volumes)
        if zone is not None:
            pulumi.set(__self__, "zone", zone)

    @_builtins.property
    @pulumi.getter(name="commercialType")
    def commercial_type(self) -> pulumi.Input[_builtins.str]:
        """
        The name of Instance commercial type.
        """
        return pulumi.get(self, "commercial_type")

    @commercial_type.setter
    def commercial_type(self, value: pulumi.Input[_builtins.str]):
        pulumi.set(self, "commercial_type", value)

    @_builtins.property
    @pulumi.getter(name="cloudInit")
    def cloud_init(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        """
        return pulumi.get(self, "cloud_init")

    @cloud_init.setter
    def cloud_init(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "cloud_init", value)

    @_builtins.property
    @pulumi.getter(name="imageId")
    def image_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        """
        return pulumi.get(self, "image_id")

    @image_id.setter
    def image_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "image_id", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The Instance group template.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="placementGroupId")
    def placement_group_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The instance placement group ID. This is optional, but it is highly recommended to set a preference for Instance location within Availability Zone.
        """
        return pulumi.get(self, "placement_group_id")

    @placement_group_id.setter
    def placement_group_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "placement_group_id", value)

    @_builtins.property
    @pulumi.getter(name="privateNetworkIds")
    def private_network_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The private Network IDs to attach to the new Instance.
        """
        return pulumi.get(self, "private_network_ids")

    @private_network_ids.setter
    def private_network_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "private_network_ids", value)

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `project_id`) The ID of the Project the Instance template is associated with.
        """
        return pulumi.get(self, "project_id")

    @project_id.setter
    def project_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "project_id", value)

    @_builtins.property
    @pulumi.getter(name="publicIpsV4Count")
    def public_ips_v4_count(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The number of flexible IPv4 addresses to attach to the new Instance.
        """
        return pulumi.get(self, "public_ips_v4_count")

    @public_ips_v4_count.setter
    def public_ips_v4_count(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "public_ips_v4_count", value)

    @_builtins.property
    @pulumi.getter(name="publicIpsV6Count")
    def public_ips_v6_count(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The number of flexible IPv6 addresses to attach to the new Instance.
        """
        return pulumi.get(self, "public_ips_v6_count")

    @public_ips_v6_count.setter
    def public_ips_v6_count(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "public_ips_v6_count", value)

    @_builtins.property
    @pulumi.getter(name="securityGroupId")
    def security_group_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The instance security group ID.
        """
        return pulumi.get(self, "security_group_id")

    @security_group_id.setter
    def security_group_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "security_group_id", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The tags associated with the Instance template.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter
    def volumes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['InstanceTemplateVolumeArgs']]]]:
        """
        The template of Instance volume.
        """
        return pulumi.get(self, "volumes")

    @volumes.setter
    def volumes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['InstanceTemplateVolumeArgs']]]]):
        pulumi.set(self, "volumes", value)

    @_builtins.property
    @pulumi.getter
    def zone(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `zone`) The zone in which the Instance template exists.
        """
        return pulumi.get(self, "zone")

    @zone.setter
    def zone(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "zone", value)


@pulumi.input_type
class _InstanceTemplateState:
    def __init__(__self__, *,
                 cloud_init: Optional[pulumi.Input[_builtins.str]] = None,
                 commercial_type: Optional[pulumi.Input[_builtins.str]] = None,
                 created_at: Optional[pulumi.Input[_builtins.str]] = None,
                 image_id: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 placement_group_id: Optional[pulumi.Input[_builtins.str]] = None,
                 private_network_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 public_ips_v4_count: Optional[pulumi.Input[_builtins.int]] = None,
                 public_ips_v6_count: Optional[pulumi.Input[_builtins.int]] = None,
                 security_group_id: Optional[pulumi.Input[_builtins.str]] = None,
                 status: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 updated_at: Optional[pulumi.Input[_builtins.str]] = None,
                 volumes: Optional[pulumi.Input[Sequence[pulumi.Input['InstanceTemplateVolumeArgs']]]] = None,
                 zone: Optional[pulumi.Input[_builtins.str]] = None):
        """
        Input properties used for looking up and filtering InstanceTemplate resources.
        :param pulumi.Input[_builtins.str] cloud_init: The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        :param pulumi.Input[_builtins.str] commercial_type: The name of Instance commercial type.
        :param pulumi.Input[_builtins.str] created_at: Date and time of Instance group's creation (RFC 3339 format).
        :param pulumi.Input[_builtins.str] image_id: The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        :param pulumi.Input[_builtins.str] name: The Instance group template.
        :param pulumi.Input[_builtins.str] placement_group_id: The instance placement group ID. This is optional, but it is highly recommended to set a preference for Instance location within Availability Zone.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] private_network_ids: The private Network IDs to attach to the new Instance.
        :param pulumi.Input[_builtins.str] project_id: `project_id`) The ID of the Project the Instance template is associated with.
        :param pulumi.Input[_builtins.int] public_ips_v4_count: The number of flexible IPv4 addresses to attach to the new Instance.
        :param pulumi.Input[_builtins.int] public_ips_v6_count: The number of flexible IPv6 addresses to attach to the new Instance.
        :param pulumi.Input[_builtins.str] security_group_id: The instance security group ID.
        :param pulumi.Input[_builtins.str] status: The Instance template status
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags associated with the Instance template.
        :param pulumi.Input[_builtins.str] updated_at: Date and time of Instance group's last update (RFC 3339 format).
        :param pulumi.Input[Sequence[pulumi.Input['InstanceTemplateVolumeArgs']]] volumes: The template of Instance volume.
        :param pulumi.Input[_builtins.str] zone: `zone`) The zone in which the Instance template exists.
        """
        if cloud_init is not None:
            pulumi.set(__self__, "cloud_init", cloud_init)
        if commercial_type is not None:
            pulumi.set(__self__, "commercial_type", commercial_type)
        if created_at is not None:
            pulumi.set(__self__, "created_at", created_at)
        if image_id is not None:
            pulumi.set(__self__, "image_id", image_id)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if placement_group_id is not None:
            pulumi.set(__self__, "placement_group_id", placement_group_id)
        if private_network_ids is not None:
            pulumi.set(__self__, "private_network_ids", private_network_ids)
        if project_id is not None:
            pulumi.set(__self__, "project_id", project_id)
        if public_ips_v4_count is not None:
            pulumi.set(__self__, "public_ips_v4_count", public_ips_v4_count)
        if public_ips_v6_count is not None:
            pulumi.set(__self__, "public_ips_v6_count", public_ips_v6_count)
        if security_group_id is not None:
            pulumi.set(__self__, "security_group_id", security_group_id)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if updated_at is not None:
            pulumi.set(__self__, "updated_at", updated_at)
        if volumes is not None:
            pulumi.set(__self__, "volumes", volumes)
        if zone is not None:
            pulumi.set(__self__, "zone", zone)

    @_builtins.property
    @pulumi.getter(name="cloudInit")
    def cloud_init(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        """
        return pulumi.get(self, "cloud_init")

    @cloud_init.setter
    def cloud_init(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "cloud_init", value)

    @_builtins.property
    @pulumi.getter(name="commercialType")
    def commercial_type(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The name of Instance commercial type.
        """
        return pulumi.get(self, "commercial_type")

    @commercial_type.setter
    def commercial_type(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "commercial_type", value)

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Date and time of Instance group's creation (RFC 3339 format).
        """
        return pulumi.get(self, "created_at")

    @created_at.setter
    def created_at(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "created_at", value)

    @_builtins.property
    @pulumi.getter(name="imageId")
    def image_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        """
        return pulumi.get(self, "image_id")

    @image_id.setter
    def image_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "image_id", value)

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The Instance group template.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "name", value)

    @_builtins.property
    @pulumi.getter(name="placementGroupId")
    def placement_group_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The instance placement group ID. This is optional, but it is highly recommended to set a preference for Instance location within Availability Zone.
        """
        return pulumi.get(self, "placement_group_id")

    @placement_group_id.setter
    def placement_group_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "placement_group_id", value)

    @_builtins.property
    @pulumi.getter(name="privateNetworkIds")
    def private_network_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The private Network IDs to attach to the new Instance.
        """
        return pulumi.get(self, "private_network_ids")

    @private_network_ids.setter
    def private_network_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "private_network_ids", value)

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `project_id`) The ID of the Project the Instance template is associated with.
        """
        return pulumi.get(self, "project_id")

    @project_id.setter
    def project_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "project_id", value)

    @_builtins.property
    @pulumi.getter(name="publicIpsV4Count")
    def public_ips_v4_count(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The number of flexible IPv4 addresses to attach to the new Instance.
        """
        return pulumi.get(self, "public_ips_v4_count")

    @public_ips_v4_count.setter
    def public_ips_v4_count(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "public_ips_v4_count", value)

    @_builtins.property
    @pulumi.getter(name="publicIpsV6Count")
    def public_ips_v6_count(self) -> Optional[pulumi.Input[_builtins.int]]:
        """
        The number of flexible IPv6 addresses to attach to the new Instance.
        """
        return pulumi.get(self, "public_ips_v6_count")

    @public_ips_v6_count.setter
    def public_ips_v6_count(self, value: Optional[pulumi.Input[_builtins.int]]):
        pulumi.set(self, "public_ips_v6_count", value)

    @_builtins.property
    @pulumi.getter(name="securityGroupId")
    def security_group_id(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The instance security group ID.
        """
        return pulumi.get(self, "security_group_id")

    @security_group_id.setter
    def security_group_id(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "security_group_id", value)

    @_builtins.property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        The Instance template status
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "status", value)

    @_builtins.property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]:
        """
        The tags associated with the Instance template.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]]):
        pulumi.set(self, "tags", value)

    @_builtins.property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        Date and time of Instance group's last update (RFC 3339 format).
        """
        return pulumi.get(self, "updated_at")

    @updated_at.setter
    def updated_at(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "updated_at", value)

    @_builtins.property
    @pulumi.getter
    def volumes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['InstanceTemplateVolumeArgs']]]]:
        """
        The template of Instance volume.
        """
        return pulumi.get(self, "volumes")

    @volumes.setter
    def volumes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['InstanceTemplateVolumeArgs']]]]):
        pulumi.set(self, "volumes", value)

    @_builtins.property
    @pulumi.getter
    def zone(self) -> Optional[pulumi.Input[_builtins.str]]:
        """
        `zone`) The zone in which the Instance template exists.
        """
        return pulumi.get(self, "zone")

    @zone.setter
    def zone(self, value: Optional[pulumi.Input[_builtins.str]]):
        pulumi.set(self, "zone", value)


@pulumi.type_token("scaleway:autoscaling/instanceTemplate:InstanceTemplate")
class InstanceTemplate(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cloud_init: Optional[pulumi.Input[_builtins.str]] = None,
                 commercial_type: Optional[pulumi.Input[_builtins.str]] = None,
                 image_id: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 placement_group_id: Optional[pulumi.Input[_builtins.str]] = None,
                 private_network_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 public_ips_v4_count: Optional[pulumi.Input[_builtins.int]] = None,
                 public_ips_v6_count: Optional[pulumi.Input[_builtins.int]] = None,
                 security_group_id: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 volumes: Optional[pulumi.Input[Sequence[pulumi.Input[Union['InstanceTemplateVolumeArgs', 'InstanceTemplateVolumeArgsDict']]]]] = None,
                 zone: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        """
        Books and manages Autoscaling Instance templates.

        ## Example Usage

        ### Basic

        ```python
        import pulumi
        import pulumiverse_scaleway as scaleway

        main = scaleway.autoscaling.InstanceTemplate("main",
            name="asg-template",
            commercial_type="PLAY2-MICRO",
            tags=[
                "terraform-test",
                "basic",
            ],
            volumes=[{
                "name": "as-volume",
                "volume_type": "sbs",
                "boot": True,
                "from_snapshot": {
                    "snapshot_id": main_scaleway_block_snapshot["id"],
                },
                "perf_iops": 5000,
            }],
            public_ips_v4_count=1,
            private_network_ids=[main_scaleway_vpc_private_network["id"]])
        ```

        ## Import

        Autoscaling Instance templates can be imported using `{zone}/{id}`, e.g.

        bash

        ```sh
        $ pulumi import scaleway:autoscaling/instanceTemplate:InstanceTemplate main fr-par-1/11111111-1111-1111-1111-111111111111
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] cloud_init: The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        :param pulumi.Input[_builtins.str] commercial_type: The name of Instance commercial type.
        :param pulumi.Input[_builtins.str] image_id: The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        :param pulumi.Input[_builtins.str] name: The Instance group template.
        :param pulumi.Input[_builtins.str] placement_group_id: The instance placement group ID. This is optional, but it is highly recommended to set a preference for Instance location within Availability Zone.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] private_network_ids: The private Network IDs to attach to the new Instance.
        :param pulumi.Input[_builtins.str] project_id: `project_id`) The ID of the Project the Instance template is associated with.
        :param pulumi.Input[_builtins.int] public_ips_v4_count: The number of flexible IPv4 addresses to attach to the new Instance.
        :param pulumi.Input[_builtins.int] public_ips_v6_count: The number of flexible IPv6 addresses to attach to the new Instance.
        :param pulumi.Input[_builtins.str] security_group_id: The instance security group ID.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags associated with the Instance template.
        :param pulumi.Input[Sequence[pulumi.Input[Union['InstanceTemplateVolumeArgs', 'InstanceTemplateVolumeArgsDict']]]] volumes: The template of Instance volume.
        :param pulumi.Input[_builtins.str] zone: `zone`) The zone in which the Instance template exists.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: InstanceTemplateArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Books and manages Autoscaling Instance templates.

        ## Example Usage

        ### Basic

        ```python
        import pulumi
        import pulumiverse_scaleway as scaleway

        main = scaleway.autoscaling.InstanceTemplate("main",
            name="asg-template",
            commercial_type="PLAY2-MICRO",
            tags=[
                "terraform-test",
                "basic",
            ],
            volumes=[{
                "name": "as-volume",
                "volume_type": "sbs",
                "boot": True,
                "from_snapshot": {
                    "snapshot_id": main_scaleway_block_snapshot["id"],
                },
                "perf_iops": 5000,
            }],
            public_ips_v4_count=1,
            private_network_ids=[main_scaleway_vpc_private_network["id"]])
        ```

        ## Import

        Autoscaling Instance templates can be imported using `{zone}/{id}`, e.g.

        bash

        ```sh
        $ pulumi import scaleway:autoscaling/instanceTemplate:InstanceTemplate main fr-par-1/11111111-1111-1111-1111-111111111111
        ```

        :param str resource_name: The name of the resource.
        :param InstanceTemplateArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(InstanceTemplateArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cloud_init: Optional[pulumi.Input[_builtins.str]] = None,
                 commercial_type: Optional[pulumi.Input[_builtins.str]] = None,
                 image_id: Optional[pulumi.Input[_builtins.str]] = None,
                 name: Optional[pulumi.Input[_builtins.str]] = None,
                 placement_group_id: Optional[pulumi.Input[_builtins.str]] = None,
                 private_network_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 project_id: Optional[pulumi.Input[_builtins.str]] = None,
                 public_ips_v4_count: Optional[pulumi.Input[_builtins.int]] = None,
                 public_ips_v6_count: Optional[pulumi.Input[_builtins.int]] = None,
                 security_group_id: Optional[pulumi.Input[_builtins.str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
                 volumes: Optional[pulumi.Input[Sequence[pulumi.Input[Union['InstanceTemplateVolumeArgs', 'InstanceTemplateVolumeArgsDict']]]]] = None,
                 zone: Optional[pulumi.Input[_builtins.str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = InstanceTemplateArgs.__new__(InstanceTemplateArgs)

            __props__.__dict__["cloud_init"] = cloud_init
            if commercial_type is None and not opts.urn:
                raise TypeError("Missing required property 'commercial_type'")
            __props__.__dict__["commercial_type"] = commercial_type
            __props__.__dict__["image_id"] = image_id
            __props__.__dict__["name"] = name
            __props__.__dict__["placement_group_id"] = placement_group_id
            __props__.__dict__["private_network_ids"] = private_network_ids
            __props__.__dict__["project_id"] = project_id
            __props__.__dict__["public_ips_v4_count"] = public_ips_v4_count
            __props__.__dict__["public_ips_v6_count"] = public_ips_v6_count
            __props__.__dict__["security_group_id"] = security_group_id
            __props__.__dict__["tags"] = tags
            __props__.__dict__["volumes"] = volumes
            __props__.__dict__["zone"] = zone
            __props__.__dict__["created_at"] = None
            __props__.__dict__["status"] = None
            __props__.__dict__["updated_at"] = None
        super(InstanceTemplate, __self__).__init__(
            'scaleway:autoscaling/instanceTemplate:InstanceTemplate',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            cloud_init: Optional[pulumi.Input[_builtins.str]] = None,
            commercial_type: Optional[pulumi.Input[_builtins.str]] = None,
            created_at: Optional[pulumi.Input[_builtins.str]] = None,
            image_id: Optional[pulumi.Input[_builtins.str]] = None,
            name: Optional[pulumi.Input[_builtins.str]] = None,
            placement_group_id: Optional[pulumi.Input[_builtins.str]] = None,
            private_network_ids: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            project_id: Optional[pulumi.Input[_builtins.str]] = None,
            public_ips_v4_count: Optional[pulumi.Input[_builtins.int]] = None,
            public_ips_v6_count: Optional[pulumi.Input[_builtins.int]] = None,
            security_group_id: Optional[pulumi.Input[_builtins.str]] = None,
            status: Optional[pulumi.Input[_builtins.str]] = None,
            tags: Optional[pulumi.Input[Sequence[pulumi.Input[_builtins.str]]]] = None,
            updated_at: Optional[pulumi.Input[_builtins.str]] = None,
            volumes: Optional[pulumi.Input[Sequence[pulumi.Input[Union['InstanceTemplateVolumeArgs', 'InstanceTemplateVolumeArgsDict']]]]] = None,
            zone: Optional[pulumi.Input[_builtins.str]] = None) -> 'InstanceTemplate':
        """
        Get an existing InstanceTemplate resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[_builtins.str] cloud_init: The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        :param pulumi.Input[_builtins.str] commercial_type: The name of Instance commercial type.
        :param pulumi.Input[_builtins.str] created_at: Date and time of Instance group's creation (RFC 3339 format).
        :param pulumi.Input[_builtins.str] image_id: The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        :param pulumi.Input[_builtins.str] name: The Instance group template.
        :param pulumi.Input[_builtins.str] placement_group_id: The instance placement group ID. This is optional, but it is highly recommended to set a preference for Instance location within Availability Zone.
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] private_network_ids: The private Network IDs to attach to the new Instance.
        :param pulumi.Input[_builtins.str] project_id: `project_id`) The ID of the Project the Instance template is associated with.
        :param pulumi.Input[_builtins.int] public_ips_v4_count: The number of flexible IPv4 addresses to attach to the new Instance.
        :param pulumi.Input[_builtins.int] public_ips_v6_count: The number of flexible IPv6 addresses to attach to the new Instance.
        :param pulumi.Input[_builtins.str] security_group_id: The instance security group ID.
        :param pulumi.Input[_builtins.str] status: The Instance template status
        :param pulumi.Input[Sequence[pulumi.Input[_builtins.str]]] tags: The tags associated with the Instance template.
        :param pulumi.Input[_builtins.str] updated_at: Date and time of Instance group's last update (RFC 3339 format).
        :param pulumi.Input[Sequence[pulumi.Input[Union['InstanceTemplateVolumeArgs', 'InstanceTemplateVolumeArgsDict']]]] volumes: The template of Instance volume.
        :param pulumi.Input[_builtins.str] zone: `zone`) The zone in which the Instance template exists.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _InstanceTemplateState.__new__(_InstanceTemplateState)

        __props__.__dict__["cloud_init"] = cloud_init
        __props__.__dict__["commercial_type"] = commercial_type
        __props__.__dict__["created_at"] = created_at
        __props__.__dict__["image_id"] = image_id
        __props__.__dict__["name"] = name
        __props__.__dict__["placement_group_id"] = placement_group_id
        __props__.__dict__["private_network_ids"] = private_network_ids
        __props__.__dict__["project_id"] = project_id
        __props__.__dict__["public_ips_v4_count"] = public_ips_v4_count
        __props__.__dict__["public_ips_v6_count"] = public_ips_v6_count
        __props__.__dict__["security_group_id"] = security_group_id
        __props__.__dict__["status"] = status
        __props__.__dict__["tags"] = tags
        __props__.__dict__["updated_at"] = updated_at
        __props__.__dict__["volumes"] = volumes
        __props__.__dict__["zone"] = zone
        return InstanceTemplate(resource_name, opts=opts, __props__=__props__)

    @_builtins.property
    @pulumi.getter(name="cloudInit")
    def cloud_init(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        """
        return pulumi.get(self, "cloud_init")

    @_builtins.property
    @pulumi.getter(name="commercialType")
    def commercial_type(self) -> pulumi.Output[_builtins.str]:
        """
        The name of Instance commercial type.
        """
        return pulumi.get(self, "commercial_type")

    @_builtins.property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> pulumi.Output[_builtins.str]:
        """
        Date and time of Instance group's creation (RFC 3339 format).
        """
        return pulumi.get(self, "created_at")

    @_builtins.property
    @pulumi.getter(name="imageId")
    def image_id(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The instance image ID. Can be an ID of a marketplace or personal image. This image must be compatible with `volume` and `commercial_type` template.
        """
        return pulumi.get(self, "image_id")

    @_builtins.property
    @pulumi.getter
    def name(self) -> pulumi.Output[_builtins.str]:
        """
        The Instance group template.
        """
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter(name="placementGroupId")
    def placement_group_id(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The instance placement group ID. This is optional, but it is highly recommended to set a preference for Instance location within Availability Zone.
        """
        return pulumi.get(self, "placement_group_id")

    @_builtins.property
    @pulumi.getter(name="privateNetworkIds")
    def private_network_ids(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        The private Network IDs to attach to the new Instance.
        """
        return pulumi.get(self, "private_network_ids")

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> pulumi.Output[_builtins.str]:
        """
        `project_id`) The ID of the Project the Instance template is associated with.
        """
        return pulumi.get(self, "project_id")

    @_builtins.property
    @pulumi.getter(name="publicIpsV4Count")
    def public_ips_v4_count(self) -> pulumi.Output[Optional[_builtins.int]]:
        """
        The number of flexible IPv4 addresses to attach to the new Instance.
        """
        return pulumi.get(self, "public_ips_v4_count")

    @_builtins.property
    @pulumi.getter(name="publicIpsV6Count")
    def public_ips_v6_count(self) -> pulumi.Output[Optional[_builtins.int]]:
        """
        The number of flexible IPv6 addresses to attach to the new Instance.
        """
        return pulumi.get(self, "public_ips_v6_count")

    @_builtins.property
    @pulumi.getter(name="securityGroupId")
    def security_group_id(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        The instance security group ID.
        """
        return pulumi.get(self, "security_group_id")

    @_builtins.property
    @pulumi.getter
    def status(self) -> pulumi.Output[_builtins.str]:
        """
        The Instance template status
        """
        return pulumi.get(self, "status")

    @_builtins.property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Sequence[_builtins.str]]]:
        """
        The tags associated with the Instance template.
        """
        return pulumi.get(self, "tags")

    @_builtins.property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> pulumi.Output[_builtins.str]:
        """
        Date and time of Instance group's last update (RFC 3339 format).
        """
        return pulumi.get(self, "updated_at")

    @_builtins.property
    @pulumi.getter
    def volumes(self) -> pulumi.Output[Optional[Sequence['outputs.InstanceTemplateVolume']]]:
        """
        The template of Instance volume.
        """
        return pulumi.get(self, "volumes")

    @_builtins.property
    @pulumi.getter
    def zone(self) -> pulumi.Output[Optional[_builtins.str]]:
        """
        `zone`) The zone in which the Instance template exists.
        """
        return pulumi.get(self, "zone")




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'GetGrafanaResult',
    'AwaitableGetGrafanaResult',
    'get_grafana',
    'get_grafana_output',
]

@pulumi.output_type
class GetGrafanaResult:
    """
    A collection of values returned by getGrafana.
    """
    def __init__(__self__, grafana_url=None, id=None, project_id=None):
        if grafana_url and not isinstance(grafana_url, str):
            raise TypeError("Expected argument 'grafana_url' to be a str")
        pulumi.set(__self__, "grafana_url", grafana_url)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if project_id and not isinstance(project_id, str):
            raise TypeError("Expected argument 'project_id' to be a str")
        pulumi.set(__self__, "project_id", project_id)

    @_builtins.property
    @pulumi.getter(name="grafanaUrl")
    def grafana_url(self) -> _builtins.str:
        """
        The URL to access the Grafana dashboard. Use your Scaleway IAM credentials to authenticate.
        """
        return pulumi.get(self, "grafana_url")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> _builtins.str:
        return pulumi.get(self, "project_id")


class AwaitableGetGrafanaResult(GetGrafanaResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetGrafanaResult(
            grafana_url=self.grafana_url,
            id=self.id,
            project_id=self.project_id)


def get_grafana(project_id: Optional[_builtins.str] = None,
                opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetGrafanaResult:
    """
    Use this data source to access information about an existing resource.

    :param _builtins.str project_id: The ID of the project the Grafana instance is associated with. If not provided, the default project configured in the provider is used.
    """
    __args__ = dict()
    __args__['projectId'] = project_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('scaleway:observability/getGrafana:getGrafana', __args__, opts=opts, typ=GetGrafanaResult).value

    return AwaitableGetGrafanaResult(
        grafana_url=pulumi.get(__ret__, 'grafana_url'),
        id=pulumi.get(__ret__, 'id'),
        project_id=pulumi.get(__ret__, 'project_id'))
def get_grafana_output(project_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                       opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetGrafanaResult]:
    """
    Use this data source to access information about an existing resource.

    :param _builtins.str project_id: The ID of the project the Grafana instance is associated with. If not provided, the default project configured in the provider is used.
    """
    __args__ = dict()
    __args__['projectId'] = project_id
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('scaleway:observability/getGrafana:getGrafana', __args__, opts=opts, typ=GetGrafanaResult)
    return __ret__.apply(lambda __response__: GetGrafanaResult(
        grafana_url=pulumi.get(__response__, 'grafana_url'),
        id=pulumi.get(__response__, 'id'),
        project_id=pulumi.get(__response__, 'project_id')))



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import builtins as _builtins
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs

__all__ = [
    'GetInstancePolicyResult',
    'AwaitableGetInstancePolicyResult',
    'get_instance_policy',
    'get_instance_policy_output',
]

@pulumi.output_type
class GetInstancePolicyResult:
    """
    A collection of values returned by getInstancePolicy.
    """
    def __init__(__self__, action=None, id=None, instance_group_id=None, instance_policy_id=None, metrics=None, name=None, priority=None, project_id=None, type=None, value=None, zone=None):
        if action and not isinstance(action, str):
            raise TypeError("Expected argument 'action' to be a str")
        pulumi.set(__self__, "action", action)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if instance_group_id and not isinstance(instance_group_id, str):
            raise TypeError("Expected argument 'instance_group_id' to be a str")
        pulumi.set(__self__, "instance_group_id", instance_group_id)
        if instance_policy_id and not isinstance(instance_policy_id, str):
            raise TypeError("Expected argument 'instance_policy_id' to be a str")
        pulumi.set(__self__, "instance_policy_id", instance_policy_id)
        if metrics and not isinstance(metrics, list):
            raise TypeError("Expected argument 'metrics' to be a list")
        pulumi.set(__self__, "metrics", metrics)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if priority and not isinstance(priority, int):
            raise TypeError("Expected argument 'priority' to be a int")
        pulumi.set(__self__, "priority", priority)
        if project_id and not isinstance(project_id, str):
            raise TypeError("Expected argument 'project_id' to be a str")
        pulumi.set(__self__, "project_id", project_id)
        if type and not isinstance(type, str):
            raise TypeError("Expected argument 'type' to be a str")
        pulumi.set(__self__, "type", type)
        if value and not isinstance(value, int):
            raise TypeError("Expected argument 'value' to be a int")
        pulumi.set(__self__, "value", value)
        if zone and not isinstance(zone, str):
            raise TypeError("Expected argument 'zone' to be a str")
        pulumi.set(__self__, "zone", zone)

    @_builtins.property
    @pulumi.getter
    def action(self) -> _builtins.str:
        return pulumi.get(self, "action")

    @_builtins.property
    @pulumi.getter
    def id(self) -> _builtins.str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @_builtins.property
    @pulumi.getter(name="instanceGroupId")
    def instance_group_id(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "instance_group_id")

    @_builtins.property
    @pulumi.getter(name="instancePolicyId")
    def instance_policy_id(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "instance_policy_id")

    @_builtins.property
    @pulumi.getter
    def metrics(self) -> Sequence['outputs.GetInstancePolicyMetricResult']:
        return pulumi.get(self, "metrics")

    @_builtins.property
    @pulumi.getter
    def name(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "name")

    @_builtins.property
    @pulumi.getter
    def priority(self) -> _builtins.int:
        return pulumi.get(self, "priority")

    @_builtins.property
    @pulumi.getter(name="projectId")
    def project_id(self) -> _builtins.str:
        return pulumi.get(self, "project_id")

    @_builtins.property
    @pulumi.getter
    def type(self) -> _builtins.str:
        return pulumi.get(self, "type")

    @_builtins.property
    @pulumi.getter
    def value(self) -> _builtins.int:
        return pulumi.get(self, "value")

    @_builtins.property
    @pulumi.getter
    def zone(self) -> Optional[_builtins.str]:
        return pulumi.get(self, "zone")


class AwaitableGetInstancePolicyResult(GetInstancePolicyResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetInstancePolicyResult(
            action=self.action,
            id=self.id,
            instance_group_id=self.instance_group_id,
            instance_policy_id=self.instance_policy_id,
            metrics=self.metrics,
            name=self.name,
            priority=self.priority,
            project_id=self.project_id,
            type=self.type,
            value=self.value,
            zone=self.zone)


def get_instance_policy(instance_group_id: Optional[_builtins.str] = None,
                        instance_policy_id: Optional[_builtins.str] = None,
                        name: Optional[_builtins.str] = None,
                        zone: Optional[_builtins.str] = None,
                        opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetInstancePolicyResult:
    """
    Gets information about an Autoscaling Instance policy.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_scaleway as scaleway

    # Get info by name (instance_group_id is required when using name)
    by_name = scaleway.autoscaling.get_instance_policy(name="my-instance-policy",
        instance_group_id=main["id"])
    # Get info by ID
    by_id = scaleway.autoscaling.get_instance_policy(instance_policy_id="11111111-1111-1111-1111-111111111111")
    ```


    :param _builtins.str instance_group_id: The ID of the Instance group the policy belongs to. Required when looking up by `name`.
    :param _builtins.str instance_policy_id: The ID of the Instance policy. Only one of `name` and `instance_policy_id` should be specified.
    :param _builtins.str name: The name of the Instance policy. Only one of `name` and `instance_policy_id` should be specified. When using `name`, `instance_group_id` is required.
    :param _builtins.str zone: `zone`) The zone in which the Instance policy exists.
    """
    __args__ = dict()
    __args__['instanceGroupId'] = instance_group_id
    __args__['instancePolicyId'] = instance_policy_id
    __args__['name'] = name
    __args__['zone'] = zone
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('scaleway:autoscaling/getInstancePolicy:getInstancePolicy', __args__, opts=opts, typ=GetInstancePolicyResult).value

    return AwaitableGetInstancePolicyResult(
        action=pulumi.get(__ret__, 'action'),
        id=pulumi.get(__ret__, 'id'),
        instance_group_id=pulumi.get(__ret__, 'instance_group_id'),
        instance_policy_id=pulumi.get(__ret__, 'instance_policy_id'),
        metrics=pulumi.get(__ret__, 'metrics'),
        name=pulumi.get(__ret__, 'name'),
        priority=pulumi.get(__ret__, 'priority'),
        project_id=pulumi.get(__ret__, 'project_id'),
        type=pulumi.get(__ret__, 'type'),
        value=pulumi.get(__ret__, 'value'),
        zone=pulumi.get(__ret__, 'zone'))
def get_instance_policy_output(instance_group_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                               instance_policy_id: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                               name: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                               zone: Optional[pulumi.Input[Optional[_builtins.str]]] = None,
                               opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetInstancePolicyResult]:
    """
    Gets information about an Autoscaling Instance policy.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_scaleway as scaleway

    # Get info by name (instance_group_id is required when using name)
    by_name = scaleway.autoscaling.get_instance_policy(name="my-instance-policy",
        instance_group_id=main["id"])
    # Get info by ID
    by_id = scaleway.autoscaling.get_instance_policy(instance_policy_id="11111111-1111-1111-1111-111111111111")
    ```


    :param _builtins.str instance_group_id: The ID of the Instance group the policy belongs to. Required when looking up by `name`.
    :param _builtins.str instance_policy_id: The ID of the Instance policy. Only one of `name` and `instance_policy_id` should be specified.
    :param _builtins.str name: The name of the Instance policy. Only one of `name` and `instance_policy_id` should be specified. When using `name`, `instance_group_id` is required.
    :param _builtins.str zone: `zone`) The zone in which the Instance policy exists.
    """
    __args__ = dict()
    __args__['instanceGroupId'] = instance_group_id
    __args__['instancePolicyId'] = instance_policy_id
    __args__['name'] = name
    __args__['zone'] = zone
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('scaleway:autoscaling/getInstancePolicy:getInstancePolicy', __args__, opts=opts, typ=GetInstancePolicyResult)
    return __ret__.apply(lambda __response__: GetInstancePolicyResult(
        action=pulumi.get(__response__, 'action'),
        id=pulumi.get(__response__, 'id'),
        instance_group_id=pulumi.get(__response__, 'instance_group_id'),
        instance_policy_id=pulumi.get(__response__, 'instance_policy_id'),
        metrics=pulumi.get(__response__, 'metrics'),
        name=pulumi.get(__response__, 'name'),
        priority=pulumi.get(__response__, 'priority'),
        project_id=pulumi.get(__response__, 'project_id'),
        type=pulumi.get(__response__, 'type'),
        value=pulumi.get(__response__, 'value'),
        zone=pulumi.get(__response__, 'zone')))
